#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::MesonAppdirConfigure
#
# This class is a subclass of SimpleAppdirConfigure. It is guaranteed to
# make use of meson..
#
# Usage example:
#
#   RBT::Action::MesonAppdirConfigure.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/meson_appdir_configure/meson_appdir_configure.rb'
# RBT::Action::MesonAppdirConfigure.new(ARGV)
# RBT.simple_appdir_configure(ARGV)
# =========================================================================== #
require 'rbt/actions/individual_actions/simple_appdir_configure/simple_appdir_configure.rb'

module RBT

class Action

class MesonAppdirConfigure < RBT::Action::SimpleAppdirConfigure # === RBT::Action::MesonAppdirConfigure

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    case run_already
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      run_already = false
    end
    run if run_already
  end
  
  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    do_use_a_build_directory
    make_use_of_meson
  end

  # ========================================================================= #
  # === RBT::Action::MesonAppdirConfigure[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.meson_appdir_configure
# =========================================================================== #
def self.meson_appdir_configure(i = ARGV)
  RBT::Action::MesonAppdirConfigure.new(i)
end; self.instance_eval { alias simple_meson_conf meson_appdir_configure } # === RBT.simple_meson_conf

end

if __FILE__ == $PROGRAM_NAME
   RBT.meson_appdir_configure(ARGV)
end # mesonappdirconfigure htop