#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/actions/individual_actions/software_manager/extract_related_code.rb'
# =========================================================================== #
module RBT

class Action

class SoftwareManager < RBT::Action # === RBT::Action::SoftwareManager

  # ========================================================================= #
  # === extract_what_to                          (extract tag, extracter tag)
  #
  # This method will extract the given archive directory, by delegating
  # towards class Extracter. This is done, since as of the year 2024,
  # via action(:extract_what_to).
  #
  # This method ought to be invoked quite early, logically; the method
  # "consider_extracting_this_program()" may be used to invoke this
  # method.
  # ========================================================================= #
  def extract_what_to(
      local_path                                                = cookbook_dataset_program_path?,
      extract_to_this_directory                                 = extract_where_to?,
      shall_we_report_to_the_user_if_the_archive_does_not_exist = true # This should come last.
    )
    append_to_the_commandline = ''.dup
    # ======================================================================== #
    # === Handle Hashes next
    # ======================================================================== #
    if local_path.is_a? Hash
      # ====================================================================== #
      # === :use_this_local_path
      # ====================================================================== #
      if local_path.has_key? :use_this_local_path
        local_path = local_path.delete(:use_this_local_path)
      end
    end
    local_path = local_path.to_s # Past this point we need a String.
    if local_path and File.exist?(local_path)
      if run_simulation?
        # =================================================================== #
        # Here we are in simulation-mode:
        # =================================================================== #
        orev "We would normally extract `#{sfile(local_path)}#{rev}` "\
             "to #{sdir(extract_to_this_directory)} #{rev}next,"
        orev 'but since we run in simulation-mode, we will not '\
             'extract anything.'
      else
        _ = "Next trying to extract `#{sfile(local_path)}#{rev}`".dup
        _ << " #{rev}(but do so silently)" unless show_output_from_the_extracter_class?
        _ << '.'
        orev _
        # =================================================================== #
        # Extracting the archive next, via an action - this will be handled
        # by class Extracter. The second argument is a Hash which will
        # typically carry a lot of information. This is necessary because
        # we want to "fine-tune" Extracter at this step.
        # =================================================================== #
        action(:extract_what_to, local_path,
          {
            extract_to:                     extract_to_this_directory, # ← Extract to this directory.
            use_colours:                    use_colours?,
            prepend_this_namespace:         'RBT→', # ← Use this modifier.
            run_simulation:                 run_simulation?,
            pad_opn_with_n_tokens:          RBT.n_padding_to_use_for_opn?,
            use_opn:                        show_the_classname?,
            be_verbose:                     show_output_from_the_extracter_class?,
            append_this_to_the_commandline: append_to_the_commandline,
          }
        ) { :be_quiet }
        # =================================================================== #
        # Next we will exit if the user told us that he/she is only
        # interested in extracting the program.
        # =================================================================== #
        exit_program if extract_only?
        return extract_to_this_directory # And return it too, in case downstream code needs it.
      end
    else # else the archive at hand does not exist.
      if shall_we_report_to_the_user_if_the_archive_does_not_exist
        opne swarn('No file could be found at `')+
             sfile(local_path)+swarn('` - thus,')
        opne swarn('we can not extract anything.')
        # =================================================================== #
        # Also set a specific error, so that we won't continue.
        # =================================================================== #
        # error_is 'We can not continue. Has the archive '\
        #          'been properly extracted?'
      end
      no_file_exists_at(local_path)
    end
  end; alias consider_extracting_this_program  extract_what_to # === consider_extracting_this_program
       alias invoke_extracter                  extract_what_to # === invoke_extracter
       alias default_extract_step              extract_what_to # === default_extract_step
       alias do_extract_the_archive            extract_what_to # === do_extract_the_archive
       alias extract_this_source_archive       extract_what_to # === extract_this_source_archive
       alias extract_the_program               extract_what_to # === extract_the_program
       alias extract_this_program              extract_what_to # === extract_this_program
       alias extract_this_archive              extract_what_to # === extract_this_archive
       alias extract                           extract_what_to # === extract
       alias extract_package                   extract_what_to # === extract_package
       alias extract_program                   extract_what_to # === extract_program
       alias extract_this_program_if_it_exists extract_what_to # === extract_this_program_if_it_exists
       alias do_extract                        extract_what_to # === do_extract
       alias default_extract                   extract_what_to # === default_extract
       alias try_to_extract_this_archive       extract_what_to # === try_to_extract_this_archive
       
end; end; end
