#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/actions/individual_actions/software_manager/query_related_methods.rb'
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class SoftwareManager < RBT::Action # === RBT::Action::SoftwareManager

  # ========================================================================= #
  # === cookbook_dataset_libraries?
  # ========================================================================= #
  def cookbook_dataset_libraries?
    cookbook_dataset?.libraries?
  end; alias libraries? cookbook_dataset_libraries? # === libraries?

  # ========================================================================= #
  # === cookbook_dataset_run_make?
  # ========================================================================= #
  def cookbook_dataset_run_make?
    cookbook_dataset?.run_make?
  end

  # ========================================================================= #
  # === cookbook_dataset_pre_make_commands?
  #
  # The premake options are the ones that are run before "make" is
  # invoked.
  # ========================================================================= #
  def cookbook_dataset_pre_make_commands?
    cookbook_dataset?.pre_make_commands?
  end

  # ========================================================================= #
  # === cookbook_dataset_symlink_headers?
  # ========================================================================= #
  def cookbook_dataset_symlink_headers?
    cookbook_dataset?.symlink_headers
  end

  # ========================================================================= #
  # === symlink_pkgconfig_files?
  # ========================================================================= #
  def symlink_pkgconfig_files?
    cookbook_dataset?.symlink_pkgconfig_files?
  end

  # ========================================================================= #
  # === cookbook_dataset_use_this_make_command?
  # ========================================================================= #
  def cookbook_dataset_use_this_make_command?
    cookbook_dataset?.use_this_make_command?
  end

  # ========================================================================= #
  # === use_this_make_command?
  #
  # This is most commonly just "make" - but it may also be "ninja" or
  # another build system.
  # ========================================================================= #
  def use_this_make_command?(
      this_make_command_is_to_be_used_if_set = 
        @internal_hash[:use_this_make_command]
    )
    _ = 'make' # Set this as the default.
    _ = cookbook_dataset_make_command? if cookbook_dataset_make_command?
    unless this_make_command_is_to_be_used_if_set.nil?
      _ = this_make_command_is_to_be_used_if_set unless this_make_command_is_to_be_used_if_set.empty?
    end
    return _ # And return it here.
  end; alias use_which_make_command? use_this_make_command? # === use_which_make_command?

  # ========================================================================= #
  # === exit_before_make_install?
  # ========================================================================= #
  def exit_before_make_install?
    @internal_hash[:exit_before_make_install]
  end

  # ========================================================================= #
  # === uses_appdir_prefix?
  #
  # This method will return true if the given input is an AppDir-prefix;
  # and false otherwise.
  #
  # An AppDir prefix is one that may include the programs directory
  # as a String - for example the string "/Programs/" or
  # "/home/Programs/".
  # ========================================================================= #
  def uses_appdir_prefix?(
      i = use_which_prefix?.to_s
    )
    return false if i.nil?
    case i
    when 'ntrad', 'appdir', 'appdir_prefix'
      return true
    else
      # ===================================================================== #
      # We may also have to check on the number, e. g.
      # (i.to_s =~ /\d+/) # And "has a number".
      # ===================================================================== #
      return i.include?(programs_directory?)
    end
  end; alias uses_an_appdir_prefix?          uses_appdir_prefix? # === uses_an_appdir_prefix?
       alias is_this_input_an_appdir_prefix? uses_appdir_prefix? # === is_this_input_an_appdir_prefix?
       alias is_this_an_appdir_prefix?       uses_appdir_prefix? # === is_this_an_appdir_prefix?
       alias is_appdir_prefix?               uses_appdir_prefix? # === is_appdir_prefix?
       alias is_an_appdir_prefix?            uses_appdir_prefix? # === is_an_appdir_prefix?
       alias app_dir_prefix?                 uses_appdir_prefix? # === app_dir_prefix?
       alias is_app_dir?                     uses_appdir_prefix? # === is_app_dir?
       alias program_uses_an_appdir_prefix?  uses_appdir_prefix? # === program_uses_an_appdir_prefix?
       alias appdir_prefix_was_used?         uses_appdir_prefix? # === appdir_prefix_was_used?

  # ========================================================================= #
  # === sed_binary_is_available?
  #
  # Determine whether the host-system has the "sed" binary available or
  # whether it has not.
  # ========================================================================= #
  def sed_binary_is_available?
    result = `sed #{SHELL_ERROR_CODE}`
    result.include? 'Usage:'
  end

  # ========================================================================= #
  # === make_file_exists?
  #
  # Determine whether a "Makefile" exists or whether it does not exist.
  # ========================================================================= #
  def make_file_exists?
    File.exist? 'Makefile'
  end; alias make_file? make_file_exists? # === make_file?

  # ========================================================================= #
  # === set_env_variables?
  #
  # Determine whether we have to set any environment variable or not.
  # ========================================================================= #
  def set_env_variables?
    _ = cookbook_dataset?
    if _ and _.respond_to?(:set_env_variables?)
      _.set_env_variables?
    else
      false
    end
  end; alias set_env? set_env_variables? # === set_env?

  # ========================================================================= #
  # === append_this_to_the_configure_options?
  # ========================================================================= #
  def append_this_to_the_configure_options?
    @internal_hash[:append_this_to_the_configure_options]
  end

  # ========================================================================= #
  # === cookbook_dataset?                                       (dataset tag)
  #
  # Keep this query method on top of this file here, ideally.
  #
  # This method can be used whenever we wish to query the main data
  # structure of the target program at hand.
  #
  # The dataset variable will typically be an instance of
  # class RBT::Cookbooks::SanitizeCookbook.
  # ========================================================================= #
  def cookbook_dataset?
    @internal_hash[:cookbook_dataset]
  end; alias cookbook_dataset   cookbook_dataset? # === cookbook_dataset
       alias cookbook_for?      cookbook_dataset? # === cookbook_for?
       alias cookbook_dataset?  cookbook_dataset? # === cookbook_dataset?
       alias dataset?           cookbook_dataset? # === dataset?
       alias dataset_object?    cookbook_dataset? # === dataset_object?
       alias main_dataset?      cookbook_dataset? # === main_dataset?
       alias main_dataset       cookbook_dataset? # === main_dataset
       alias dataset            cookbook_dataset? # === dataset
       alias dataset?           cookbook_dataset? # === cookbooks_dataset?
       alias yaml_dataset?      cookbook_dataset? # === yaml_dataset?
       alias cookbook?          cookbook_dataset? # === cookbook?
       alias cookbooks_for?     cookbook_dataset? # === cookbooks_dataset?
       alias data?              cookbook_dataset? # === data
       alias sload              cookbook_dataset? # === sload() (sload tag)
       alias sanitized_data     cookbook_dataset? # === sanitized_data()
       alias yaml_dataset?      cookbook_dataset? # === yaml_dataset?
       alias sanitize_cookbook? cookbook_dataset? # === sanitize_cookbook?
       alias my_program         cookbook_dataset? # === my_program
       alias cookbook_for?      cookbook_dataset? # === cookbook_for?

  # ========================================================================= #
  # === cookbook_dataset_has_been_initialized?
  # ========================================================================= #
  def cookbook_dataset_has_been_initialized?
    cookbook_dataset?.nil?
  end

  # ========================================================================= #
  # === scons_file_exists?
  # ========================================================================= #
  def scons_file_exists?
    File.exist? 'SConstruct'
  end; alias scons_file? scons_file_exists? # === scons_file?

  # ========================================================================= #
  # === do_we_use_any_prefix?
  # ========================================================================= #
  def do_we_use_any_prefix?
    @internal_hash[:use_a_prefix]
  end

  # ========================================================================= #
  # === use_porg?
  # ========================================================================= #
  def use_porg?
    @internal_hash[:use_porg]
  end; alias will_we_use_porg? use_porg? # === will_we_use_porg?

  # ========================================================================= #
  # === parameters_to_make_as_string?
  # ========================================================================= #
  def parameters_to_make_as_string?
    if @internal_hash[:parameters_to_make]
      @internal_hash[:parameters_to_make].join(' ').strip.squeeze(' ')
    else
      ''
    end
  end

  # ========================================================================= #
  # === dataset_program_name_and_program_version?
  #
  # This method will always return the program name and the associated
  # program version of any given program, such as:
  #
  #   "htop-2.0.2"
  #
  # ========================================================================= #
  def dataset_program_name_and_program_version?(
      cookbook_dataset = cookbook_dataset?
    )
    if cookbook_dataset
      cookbook_dataset.program_name_and_program_version? # Delegate to cookbook_for? here.
    else
      nil
    end
  end; alias cookbooks_program_name?                            dataset_program_name_and_program_version? # === cookbooks_program_name?
       alias cookbook_dataset_program_name_and_program_version? dataset_program_name_and_program_version? # === cookbook_dataset_program_name_and_program_version?
       alias cookbook_dataset_program_name_and_program_version  dataset_program_name_and_program_version? # === cookbook_dataset_program_name_and_program_version
       alias program_name_and_version?                          dataset_program_name_and_program_version? # === program_name_and_version?
       alias program_name_and_program_version?                  dataset_program_name_and_program_version? # === program_name_and_program_version?

  # ========================================================================= #
  # === cookbook_installation_steps?
  #
  # The installation-steps contain the instructions in how to install
  # a given program at hand.
  # ========================================================================= #
  def cookbook_installation_steps?
    cookbook_dataset?.installation_steps?
  end; alias installation_steps? cookbook_installation_steps? # === cookbook_installation_steps?

  # ========================================================================= #
  # === cookbooks_dataset_extracted_towards?
  # ========================================================================= #
  def cookbooks_dataset_extracted_towards?
    if cookbooks_dataset?
      cookbooks_dataset?.extracted_towards?
    else
      rbt_log_directory?
    end
  end

  # ========================================================================= #
  # === depends_on_python?
  #
  # Whether python is registered as a dependency.
  # ========================================================================= #
  def depends_on_python?
    RBT.depends_on_python?(required_deps_on?)
  end

  # ========================================================================= #
  # === depends_on_waf?
  # ========================================================================= #
  def depends_on_waf?
    ::RBT.depends_on_waf?
  end

  # ========================================================================= #
  # === cookbook_dataset_flatpak_url?
  # ========================================================================= #
  def cookbook_dataset_flatpak_url?
    cookbook_dataset?.flatpak_url
  end; alias flatpak_url? cookbook_dataset_flatpak_url? # === flatpak_url?
       alias flatpak?     cookbook_dataset_flatpak_url? # === flatpak?

  # ========================================================================= #
  # === has_flatpak_url?
  # ========================================================================= #
  def has_flatpak_url?
    !cookbook_dataset?.flatpak_url.empty?
  end

  # ========================================================================= #
  # === configure_has_a_higher_priority_than_meson?
  #
  # This method will return true if "configure" has a higher priority
  # than "meson". A yaml file keeps track of these priorities.
  # ========================================================================= #
  def configure_has_a_higher_priority_than_meson?
    ::RBT.configure_has_a_higher_priority_than_meson?
  end

  # ========================================================================= #
  # === cookbooks_dataset_use_glib_schema?
  #
  # Whether our dataset uses glib install schema for some .xml files.
  # ========================================================================= #
  def cookbooks_dataset_use_glib_schema?
    cookbook_dataset?.use_glib_schema
  end; alias use_glib_schema? cookbooks_dataset_use_glib_schema? # === use_glib_schema?

  # ========================================================================= #
  # === dataset_program_short_name?
  #
  # The short name would be "gnomemusic" if the regular name is
  # "gnome-music".
  # ========================================================================= #
  def dataset_program_short_name?
    cookbook_dataset?.short_name?
  end; alias dataset_program_name?               dataset_program_short_name? # === dataset_program_name?
       alias cookbook_dataset_real_program_name? dataset_program_short_name? # === cookbook_dataset_real_program_name?
       alias real_short_name?                    dataset_program_short_name? # === real_short_name?

  # ========================================================================= #
  # === required_deps_on?
  # ========================================================================= #
  def required_deps_on?
    cookbook_dataset?.required_deps_on?
  end; alias required_deps?            required_deps_on? # === required_deps?
       alias required_deps             required_deps_on? # === required_deps
       alias required_dependencies?    required_deps_on? # === required_dependencies?
       alias required_dependencies_on? required_deps_on? # === required_dependencies_on?

  # ========================================================================= #
  # === make_object?
  #
  # Query-method towards the @make object (aka RBT::Make).
  # ========================================================================= #
  def make_object?
    @internal_hash[:make]
  end

  # ========================================================================= #
  # === sed?
  #
  # Query as to whether we will use sed or not.
  # ========================================================================= #
  def sed?
    _ = cookbook_dataset?.sed?
    if @internal_hash[:sed].nil?
      # ===================================================================== #
      # In this case we will use the sed-variant given by the cookbooks
      # dataset.
      # ===================================================================== #
    else
      _ = @internal_hash[:sed]
    end
    return _
  end

  # ========================================================================= #
  # === blfs?
  # ========================================================================= #
  def blfs?
    dataset?.blfs?
  end

  # ========================================================================= #
  # === do_not_compile?
  # ========================================================================= #
  def do_not_compile?
    @internal_hash[:do_not_compile]
  end

  # ========================================================================= #
  # === binary?
  #
  # Wrapper towards answering the question of binary pointing.
  # ========================================================================= #
  def binary?(
      i = compile_which_program?
    )
    i = i.first if i.is_a? Array
    i = i.downcase
    orev "Checking for a binary name called `#{simp(i)}#{rev}` next:"
    return_program_based_on_this_binary(i)
  end

  # ========================================================================= #
  # === will_gcc_be_used_for_compilation?
  # ========================================================================= #
  def will_gcc_be_used_for_compilation?
    use_which_compiler? == 'gcc'
  end

  # ========================================================================= #
  # === has_the_program_been_installed_successfully?
  # ========================================================================= #
  def has_the_program_been_installed_successfully?
    @internal_hash[:has_the_program_been_installed_successfully]
  end; alias successfully_compiled?          has_the_program_been_installed_successfully? # === successfully_compiled?
       alias has_been_successfully_compiled? has_the_program_been_installed_successfully? # === has_been_successfully_compiled?
       alias installation_was_successful?    has_the_program_been_installed_successfully? # === installation_was_successful?
       alias compiled_successfully?          has_the_program_been_installed_successfully? # === compiled_successfully?

  # ========================================================================= #
  # === is_this_program_included?
  # ========================================================================= #
  def is_this_program_included?(
      i,
      be_verbose = be_verbose?
    )
    RBT::Action::SoftwareManager.is_this_program_included?(
      i, be_verbose
    )
  end
       
  # ========================================================================= #
  # === env?
  # ========================================================================= #
  def env?
    ENV
  end

  # ========================================================================= #
  # === cookbook_dataset_program_path?
  # ========================================================================= #
  def cookbook_dataset_program_path?(
      i = cookbook_dataset?
    )
    if i
      i.local_path
    end
  end; alias cookbook_dataset_path?  cookbook_dataset_program_path? # === cookbook_dataset_path?
       alias full_path?              cookbook_dataset_program_path? # === full_path?
       alias cookbooks_program_path? cookbook_dataset_program_path? # === cookbooks_program_path?
       alias cookbook_program_path?  cookbook_dataset_program_path? # === cookbook_program_path?
       alias program_path?           cookbook_dataset_program_path? # === program_path?

  # ========================================================================= #
  # === cookbook_dataset_headers?
  # ========================================================================= #
  def cookbook_dataset_headers?
    cookbook_dataset?.headers?
  end; alias headers? cookbook_dataset_headers? # === headers?

  # ========================================================================= #
  # === is_a_kde_program?
  #
  # We assume that every program that has the "kde" tag associated,
  # is a KDE program. Thus this method will return true.
  # ========================================================================= #
  def is_a_kde_program?
    tags.include? 'kde'
  end

  # ========================================================================= #
  # === use_which_prefix?
  #
  # This will properly handle the prefix in use. It is the general
  # getter-method which should be used whenever we wish to obtain
  # the prefix in use.
  # ========================================================================= #
  def use_which_prefix?
    _ = prefix_object? # .prefix? # @internal_hash[:prefix]
    if _.do_we_want_to_use_an_appdir_prefix?
      _.use_appdir_prefix_for_this_program(compile_which_program?)
    end
    return _
  end; alias prefix?               use_which_prefix? # === prefix?
       alias prefix                use_which_prefix? # === prefix
       alias user_prefix?          use_which_prefix? # === user_prefix?
       alias return_proper_prefix  use_which_prefix? # === return_proper_prefix
       alias use_this_prefix?      use_which_prefix? # === use_this_prefix?
       alias real_prefix?          use_which_prefix? # === real_prefix?
       alias return_proper_prefix? use_which_prefix? # === return_proper_prefix?
       alias return_proper_prefix  use_which_prefix? # === return_proper_prefix

  # ========================================================================= #
  # === action_chained_programs?
  # ========================================================================= #
  def action_chained_programs?
    action(:chained_programs) # Load the chained actions, then return it.
  end

  # ========================================================================= #
  # === use_this_as_build_directory?
  # ========================================================================= #
  def use_this_as_build_directory?
    @internal_hash[:use_this_as_build_directory]
  end

  # ========================================================================= #
  # === will_be_extracted_towards?
  #
  # This method will tell us the full path of the extracted directory for
  # the program at hand.
  #
  # A trailing '/' is required for the result of this method.
  #
  # Returns: a String.
  # ========================================================================= #
  def will_be_extracted_towards?
    "#{log_directory?}#{program_name_and_program_version?}/"
  end

  # ========================================================================= #
  # === cookbook_aliases?
  # ========================================================================= #
  def cookbook_aliases?
    @internal_hash[:cookbook_aliases]
  end

  # ========================================================================= #
  # === is_on_gobolinux?
  #
  # Query whether we are on GoboLinux or whether we are not.
  #
  # In the past this was only a check for an environment variable.
  #
  # Since we will allow commandline options to change this, we also had
  # to use a proper variable to keep track of this.
  # ========================================================================= #
  def is_on_gobolinux?
    @internal_hash[:is_on_gobolinux]
  end

  # ========================================================================= #
  # === last_system_command_was_a_success?
  # ========================================================================= #
  def last_system_command_was_a_success?
    $CHILD_STATUS.success?
  end; alias was_the_configure_run_a_success? last_system_command_was_a_success? # === was_the_configure_run_a_success?
       alias show_success_status?             last_system_command_was_a_success? # === show_success_status?

  # ========================================================================= #
  # === use_which_system_command?
  # ========================================================================= #
  def use_which_system_command?
    @internal_hash[:use_which_system_command]
  end

  # ========================================================================= #
  # === last_error_line?
  # ========================================================================= #
  def last_error_line?
    sys_command_string?.last
  end

  # ========================================================================= #
  # === pre_make_install_sed?
  # ========================================================================= #
  def pre_make_install_sed?
    cookbook_dataset?.pre_make_install_sed?
  end; alias pre_make_install_sed pre_make_install_sed? # === pre_make_install_sed

  # ========================================================================= #
  # === symlink_glib_schema?
  # ========================================================================= #
  def symlink_glib_schema?
    _ = cookbooks_dataset_use_glib_schema?
    unless @internal_hash[:symlink_glib_schema].nil? 
      _ = @internal_hash[:symlink_glib_schema]
    end
    return _
  end

  # ========================================================================= #
  # === skip_extracting?
  #
  # Query-method over as to whether we will skip extracting the archive
  # at hand. This can usually only be done when the archive has already
  # been extracted somehow.
  # ========================================================================= #
  def skip_extracting?
    _ = @internal_hash[:skip_extracting]
    _ == true # This check prohibits nil and false.
  end; alias skip_extracting_the_archive? skip_extracting? # === skip_extracting_the_archive?

  # ========================================================================= #
  # === may_we_download?
  # ========================================================================= #
  def may_we_download?
    return ihash?[:may_we_download]
  end

  # ========================================================================= #
  # === prefix_as_string?
  # ========================================================================= #
  def prefix_as_string?
    prefix?.to_s
  end

  # ========================================================================= #
  # === make_install_stage_was_a_success?
  # ========================================================================= #
  def make_install_stage_was_a_success?
    @internal_hash[:make_install_stage_was_a_success]
  end

  # ========================================================================= #
  # === an_error_has_occurred?
  #
  # Query-method over as to whether an error has occurred. 
  # ========================================================================= #
  def an_error_has_occurred?
    result = @internal_hash[:an_error_has_occurred]
    ignore_errors = ignore_errors?
    case ignore_errors
    # ======================================================================= #
    # === :infer
    # ======================================================================= #
    when :infer
      ignore_errors = cookbook_dataset_ignore_errors?
    end
    result = false if ignore_errors
    result = false if result.is_a?(Array) and result.empty?
    return result
  end; alias has_an_error_occurred?     an_error_has_occurred? # === has_an_error_occurred?
       alias did_we_encounter_an_error? an_error_has_occurred? # === did_we_encounter_an_error?

  # ========================================================================= #
  # === no_cookbook_dataset?
  # ========================================================================= #
  def no_cookbook_dataset?
    !(
      @internal_hash and 
      @internal_hash[:yaml_dataset] and 
     !@internal_hash[:yaml_dataset].nil?
    )
  end

  # ========================================================================= #
  # === full_configure_command?
  #
  # The @full_configure_command contains the full configure command
  # that we will use. The advantage in using this is that we can
  # easily copy/paste this at a later time, should we need to do so.
  #
  # We can also store this into a local file, such as a .md file.
  # In fact, this was the primary reason why it was added.
  # ========================================================================= #
  def full_configure_command?
    @internal_hash[:full_configure_command]
  end; alias return_full_configure_command full_configure_command? # === return_full_configure_command

  # ========================================================================= #
  # === check_whether_the_given_input_is_a_file_and_does_exist?
  # ========================================================================= #
  def check_whether_the_given_input_is_a_file_and_does_exist?
    @internal_hash[:check_whether_the_given_input_is_a_file_and_does_exist]
  end; alias do_we_check_whether_the_program_exists? check_whether_the_given_input_is_a_file_and_does_exist? # === do_we_check_whether_the_program_exists?

  # ========================================================================= #
  # === cookbook_dataset_prefix?
  #
  # This method name may be a slight misnomer, since class Prefix may
  # handle the prefix here.
  # ========================================================================= #
  def cookbook_dataset_prefix?
    if cookbook_dataset?
      cookbook_dataset?.prefix?
    else
      nil
    end
  end; alias cookbook_prefix?                           cookbook_dataset_prefix? # === cookbook_prefix?
       alias cookbooks_prefix?                          cookbook_dataset_prefix? # === cookbooks_prefix?
       alias dataset_prefix?                            cookbook_dataset_prefix? # === dataset_prefix?
       alias determine_prefix_from_the_cookbook_dataset cookbook_dataset_prefix? # === determine_prefix_from_the_cookbook_dataset

  # ========================================================================= #
  # === feedback_what_we_are_currently_compiling?
  #
  # This method will feedback what we are currently compiling.
  #
  # To invoke this method, do:
  #
  #   rbt --what-are-we-currently-compiling?
  #
  # ========================================================================= #
  def feedback_what_we_are_currently_compiling?(
      status_file = FILE_STATUS
    )
    if File.exist? status_file
      dataset = YAML.load_file(status_file)
    end
    this_key = 'currently_compiling_these_programs'
    if dataset.has_key?(this_key) and
       !dataset[this_key].empty? # This should work because it is an Array.
      these_programs_are_currently_compiled = dataset[this_key]
      orev 'The following programs are currently compiled:'
      e
      these_programs_are_currently_compiled.each {|this_program|
        e "  #{sfancy(this_program)}"
      }; e
    else
      orev 'No program is being currently compiled.'
    end
  end

  # ========================================================================= #
  # === honour_check_to_ensure_that_the_program_exists?
  # ========================================================================= #
  def honour_check_to_ensure_that_the_program_exists?
    @internal_hash[:honour_check_to_ensure_that_the_program_exists] # This is true by default.
  end

  # ========================================================================= #
  # === run_make_install?
  #
  # Query whether we will do the "make install" step or whether we
  # will not. Will return a boolean value (true/false).
  # ========================================================================= #
  def run_make_install?
    @internal_hash[:run_make_install]
  end; alias may_we_run_make_install_command? run_make_install? # == may_we_run_make_install_command?

  # ========================================================================= #
  # === strip_binaries?
  # ========================================================================= #
  def strip_binaries?
    @internal_hash[:strip_binaries]
  end

  # ========================================================================= #
  # === do_not_strip_binaries?
  # ========================================================================= #
  def do_not_strip_binaries?
    !strip_binaries?
  end; alias do_not_strip_the_binaries? do_not_strip_binaries? # === do_not_strip_the_binaries?

  # ========================================================================= #
  # === depends_on_ninja?
  # ========================================================================= #
  def depends_on_ninja?
    ::RBT.depends_on_ninja?(required_deps_on?)
  end

  # ========================================================================= #
  # === original_start_directory?
  # ========================================================================= #
  def original_start_directory?
    @internal_hash[:original_start_directory]
  end; alias original_base_directory? original_start_directory? # === original_base_directory?

  # ========================================================================= #
  # === run_ldconfig?
  # ========================================================================= #
  def run_ldconfig?
    RBT.configuration?.run_ldconfig
  end

  # ========================================================================= #
  # === append_shell_error_code?
  # ========================================================================= #
  def append_shell_error_code?
    @internal_hash[:append_shell_error_code]
  end

  # ========================================================================= #
  # === use_the_internal_sed_wrapper?
  #
  # Whether we will use the internal sed wrapper, aka class SedWrapper.
  #
  # Since as of February 2018, we will not do so.
  #
  # You can query this via the commandline like so:
  #   rbt use-the-internal-sed-wrapper?
  # ========================================================================= #
  def use_the_internal_sed_wrapper?
    @internal_hash[:use_the_internal_sed_wrapper]
  end

  # ========================================================================= #
  # === do_the_dependencies_include_this_program?
  #
  # This method can be used to check whether the dependencies include a
  # given program at hand (the input).
  #
  # Keep in mind that dependencies can be written in either "name" or
  # "name >= 1.2.3" style. So we also have to check for the second
  # possibility there.
  # ========================================================================= #
  def do_the_dependencies_include_this_program?(
      i = compile_which_program?
    )
    dependencies = required_dependencies_on?
    # ======================================================================= #
    # Simplify the dependencies next.
    # ======================================================================= #
    dependencies.map! {|entry|
      if entry.include?(' >') or entry.include?(' <')
        entry = entry.split(' ').first.strip
      end
      entry
    }
    dependencies.include? i
  end

  # ========================================================================= #
  # === git_url?
  # ========================================================================= #
  def git_url?
    cookbook_dataset?.git_url?
  end

  # ========================================================================= #
  # === short_description?
  # ========================================================================= #
  def short_description?
    cookbook_dataset?.short_description?
  end

  # ========================================================================= #
  # === description?
  # ========================================================================= #
  def description?
    cookbook_dataset?.description?
  end

  # ========================================================================= #
  # === cookbook_dataset_short_name?
  # ========================================================================= #
  def cookbook_dataset_short_name?
    cookbook_dataset?.short_name?.to_s
  end; alias short_name? cookbook_dataset_short_name? # === short_name?

  # ========================================================================= #
  # === install_which_programs?
  # ========================================================================= #
  def install_which_programs?
    ::RBT.install_which_programs?
  end; alias array_compile_these_programs? install_which_programs? # === array_compile_these_programs?
       alias array?                        install_which_programs? # === array?
       alias which_program?                install_which_programs? # === which_program?

  # ========================================================================= #
  # === do_not_symlink?
  # ========================================================================= #
  def do_not_symlink?
    @internal_hash[:do_not_symlink]
  end

  # ========================================================================= #
  # === compile_these_programs?
  # ========================================================================= #
  def compile_these_programs?
    @internal_hash[:compile_these_programs]
  end; alias compile_which_programs? compile_these_programs? # === compile_which_programs?
       alias compile_array?          compile_these_programs? # === compile_array?

  # ========================================================================= #
  # === has_a_dependency_on_meson?
  # ========================================================================= #
  def has_a_dependency_on_meson?
    cookbook_dataset?.has_a_dependency_on_meson?
  end

  # ========================================================================= #
  # === extract_only?
  # ========================================================================= #
  def extract_only?
    @internal_hash[:extract_only]
  end

  # ========================================================================= #
  # === cookbook_dataset_apply_patch?
  # ========================================================================= #
  def cookbook_dataset_apply_patch?
    cookbook_dataset?.apply_patch?
  end; alias apply_patch? cookbook_dataset_apply_patch? # === apply_patch?

  # ========================================================================= #
  # === cookbook_dataset_make_command?
  # ========================================================================= #
  def cookbook_dataset_make_command?
    cookbook_dataset?.use_which_make_command?
  end

  # ========================================================================= #
  # === configuration_setting_for_symlink_gir_files?
  # ========================================================================= #
  def configuration_setting_for_symlink_gir_files?
    RBT.configuration? and 
    RBT.configuration?.symlink_gir_files
  end

  # ========================================================================= #
  # === time_the_compilation?
  # ========================================================================= #
  def time_the_compilation?
    @internal_hash[:time_compilation]
  end

  # ========================================================================= #
  # === use_autogen?
  #
  # Query method over as to whether we will use autogen.
  # ========================================================================= #
  def use_autogen?
    _ = cookbook_dataset_use_autogen? 
    unless @internal_hash[:use_autogen].nil?
      _ = @internal_hash[:use_autogen]
    end
    _
  end

  # ========================================================================= #
  # === use_which_compiler?
  # ========================================================================= #
  def use_which_compiler?
    config?.use_this_compiler
  end; alias compiler_to_be_used? use_which_compiler? # === compiler_to_be_used?

  # ========================================================================= #
  # === modify_the_makefile?
  #
  # If this is used then it should be used before invoking the "make" step.
  # ========================================================================= #
  def modify_the_makefile?
    cookbook_dataset?.modify_the_makefile?
  end

  # ========================================================================= #
  # === cookbook_dataset_use_autogen?
  #
  # Query-method over as to the cookbook-dataset has an entry called
  # use_autogen set to true.
  # ========================================================================= #
  def cookbook_dataset_use_autogen?
    cookbook_dataset?.use_autogen?
  end

  # ========================================================================= #
  # === use_which_build_system?
  # ========================================================================= #
  def use_which_build_system?
    # ======================================================================= #
    # We default to the cookbook dataset when it comes to a specific
    # build system.
    # ======================================================================= #
    _ = cookbookset_dataset_use_which_build_system?
    unless @internal_hash[:use_this_build_system].nil?
      _ = @internal_hash[:use_this_build_system]
    end
    return _.to_sym
  end; alias build_system_to_be_used? use_which_build_system? # === build_system_to_be_used?
       alias build_system?            use_which_build_system? # === build_system?
       alias build_type?              use_which_build_system? # === build_type?

  # ========================================================================= #
  # === has_key?
  #
  # This is a substitute method, to enable hash-like behaviour for
  # some keys. Right now (December 2019) this is a hardcoded Array,
  # but in the future the code may be changed - let's see.
  #
  # Do note that this method is specifically synced (closely) with
  # class RBT::CreatePkgconfigFile.new. 
  # ========================================================================= #
  def has_key?(i)
    %i(
      description
      prefix
      program_version
    ).include? i
  end

  # ========================================================================= #
  # === is_registered?
  #
  # Determine whether a given program is registered or whether it is not.
  # ========================================================================= #
  def is_registered?(i)
    return RBT.is_available? i
  end; alias is_this_program_registered? is_registered? # === is_this_program_registered?

  # ========================================================================= #
  # === work_on_which_program?
  #
  # This method is a bit different to the method called install_which_programs?
  # in that it will always return the first element of the Array, whereas
  # the other method mentioned will yield an Array.
  # ========================================================================= #
  def work_on_which_program?
    ::RBT.install_which_programs?.first
  end; alias work_on_this_program? work_on_which_program? # === work_on_this_program?

  # ========================================================================= #
  # === shall_we_extract_the_archive?
  #
  # Query method over as to whether RBT will extract the archive or not.
  # ========================================================================= #
  def shall_we_extract_the_archive?
    @internal_hash[:extract_the_archive]
  end; alias extract_the_archive? shall_we_extract_the_archive? # === extract_the_archive?

  # ========================================================================= #
  # === cookbook_dataset_configure_base_directory?
  #
  # We ask the cookbook-dataset object for the configure base directory.
  #
  # Most programs will NOT have such a setting.
  # ========================================================================= #
  def cookbook_dataset_configure_base_directory?
    cookbook_dataset?.configure_base_directory?
  end; alias cookbook_dataset_configure_base_dir? cookbook_dataset_configure_base_directory? # === cookbook_dataset_configure_base_dir?

  # ========================================================================= #
  # === extract_to_this_directory_being_one_directory_below?
  #
  # This points to the base-directory.
  # ========================================================================= #
  def extract_to_this_directory_being_one_directory_below?
    _ = extract_to_this_directory?.to_s.dup
    _ = "#{File.dirname(_)}/"
    return _
  end

  # ========================================================================= #
  # === cookbook_dataset_cmake_configure_options?
  #
  # This method will return all options used for the "cmake" build
  # system of the program at hand, should cmake be used at all.
  #
  # If GNU configure is used then this method will not be used.
  # ========================================================================= #
  def cookbook_dataset_cmake_configure_options?
    cookbook_dataset?.cmake_configure_options?
  end; alias cmake_configure_options? cookbook_dataset_cmake_configure_options? # === cmake_configure_options?

  # ========================================================================= #
  # === extracted_base_directory?
  #
  # If you need to query the value for the path to the extracted
  # directory, then make use of the method extract_to_this_directory?
  # rather than the method here.
  # ========================================================================= #
  def extracted_base_directory?
    @internal_hash[:extracted_base_directory]
  end

  # ========================================================================= #
  # === compiled_programs?
  #
  # This will grab the full dataset of compiled_programs.yml
  # ========================================================================= #
  def compiled_programs?(
      from_this_file = file_compiled_programs?
    )
    if File.exist? from_this_file
      return load_yaml(from_this_file)
    else
      orev "The file at #{sfile(from_this_file)} #{rev}does not exist."
    end
  end

  # ========================================================================= #
  # === guess_build_type
  # ========================================================================= #
  def guess_build_type
    RBT.build_type?
  end; alias do_guess_the_build_type guess_build_type # === do_guess_the_build_type

  # ========================================================================= #
  # === gir_files?
  # ========================================================================= #
  def gir_files?
    cookbook_dataset?.gir_files
  end

  # ========================================================================= #
  # === pre_configure_steps?
  # ========================================================================= #
  def pre_configure_steps?
    cookbook_dataset?.pre_configure_steps?
  end

  # ========================================================================= #
  # === pkgconfig_files?
  #
  # Query method over the registered .pc files of a particular program.
  # ========================================================================= #
  def pkgconfig_files?
    cookbook_dataset?.pkgconfig_files?
  end; alias pc_files? pkgconfig_files? # === pc_files?

  # ========================================================================= #
  # === main_eight?
  # ========================================================================= #
  def main_eight?
    cookbook_dataset?.main_eight?
  end

  # ========================================================================= #
  # === program_compile_name?
  # ========================================================================= #
  def program_compile_name?
    cookbook_dataset?.program_compile_name?.to_s
  end

  # ========================================================================= #
  # === array_sys_command_string?
  #
  # The array is fed in line by line.
  # ========================================================================= #
  def array_sys_command_string?
    @internal_hash[:array_sys_command_string]
  end; alias sys_command_string? array_sys_command_string? # === sys_command_string?

  # ========================================================================= #
  # === exit_after_a_rubygem_has_been_installed?
  # ========================================================================= #
  def exit_after_a_rubygem_has_been_installed?
    @internal_hash[:exit_after_a_rubygem_has_been_installed]
  end

  # ========================================================================= #
  # === depends_on_cmake?
  # ========================================================================= #
  def depends_on_cmake?
    RBT.depends_on_cmake?(required_deps_on?)
  end; alias program_uses_cmake? depends_on_cmake? # === program_uses_cmake?

  # ========================================================================= #
  # === cmake_file_exists?
  # ========================================================================= #
  def cmake_file_exists?
    File.exist? 'CMakeLists.txt'
  end; alias cmake_file? cmake_file_exists? # === cmake_file?

  # ========================================================================= #
  # === try_to_determine_which_programs_have_to_be_installed?
  # ========================================================================= #
  def try_to_determine_which_programs_have_to_be_installed?
    @internal_hash[:try_to_determine_which_programs_have_to_be_installed]
  end

  # ========================================================================= #
  # === rename_tab_title?
  # ========================================================================= #
  def rename_tab_title?
    @internal_hash[:rename_tab_title]
  end; alias rename_the_terminal? rename_tab_title? # === rename_the_terminal?

  # ========================================================================= #
  # === dataset_from_the_file_cookbook_chained_programs?
  #
  # This query-method obtains the dataset from the yaml file containing
  # all chained-programs.
  #
  # The alias-name .chained_programs?() makes accessing this information
  # even simpler.
  # ========================================================================= #
  def dataset_from_the_file_cookbook_chained_programs?
    @internal_hash[:dataset_from_the_file_cookbook_chained_programs]
  end; alias chained_programs? dataset_from_the_file_cookbook_chained_programs? # === chained_programs?

  # ========================================================================= #
  # === try_to_recompile_upon_error?
  # ========================================================================= #
  def try_to_recompile_upon_error?
    @internal_hash[:try_to_recompile_upon_error]
  end

  # ========================================================================= #
  # === may_we_initialize_a_new_cookbook_dataset?
  # ========================================================================= #
  def may_we_initialize_a_new_cookbook_dataset?
    @internal_hash[:may_we_initialize_a_new_cookbook_dataset]
  end

  # ========================================================================= #
  # === configure_options?
  #
  # This method will retrieve the configure options. We default to using
  # the configure-options stored in the cookbook dataset, but we can
  # overrule this on the commandline (when we store it onto the
  # @internal_hash).
  # ========================================================================= #
  def configure_options?(
      _ = cookbook_dataset_configure_options?
    )
    if @internal_hash[:configure_options]
      _ = @internal_hash[:configure_options]
    end
    return _
  end

  # ========================================================================= #
  # === cookbooks_dataset_configure_options?
  # ========================================================================= #
  def cookbooks_dataset_configure_options?
    cookbook_dataset?.configure_options?
  end; alias cookbook_dataset_configure_options? cookbooks_dataset_configure_options? # === cookbook_dataset_configure_options?

  # ========================================================================= #
  # === may_we_run_ninja?
  # ========================================================================= #
  def may_we_run_ninja?
    @internal_hash[:may_we_run_ninja]
  end

  # ========================================================================= #
  # === homepage?
  # ========================================================================= #
  def homepage?(cookbook_dataset = cookbook_dataset?)
    if cookbook_dataset
      cookbook_dataset.homepage?
    end
  end

  # ========================================================================= #
  # === program_full_name?
  #
  # This will always query the dataset object.
  # ========================================================================= #
  def program_full_name?
    cookbook_dataset?.program_full_name? # This returns e. g. 'gnome-mahjonng-5.08.tar.xz'
  end

  # ========================================================================= #
  # === highest_priority
  #
  # This method will return the main build system to use in general,
  # if possible.
  # ========================================================================= #
  def highest_priority?
    return_build_systems_priorities.first
  end

  # ========================================================================= #
  # === symlink_gir_files?
  # ========================================================================= #
  def symlink_gir_files?
    @internal_hash[:symlink_gir_files]
  end

  # ========================================================================= #
  # === is_meson_build_type?
  #
  # This method will return true if we can use meson in the current
  # directory; and false otherwise.
  # ========================================================================= #
  def is_meson_build_type?
    RBT.build_type? == :meson
  end; alias uses_meson? is_meson_build_type? # === uses_meson?

  # ========================================================================= #
  # === make_use_of_cookbook_aliases?
  # ========================================================================= #
  def make_use_of_cookbook_aliases?
    @internal_hash[:make_use_of_cookbook_aliases]
  end

  # ========================================================================= #
  # === show_output_from_the_extracter_class?
  # ========================================================================= #
  def show_output_from_the_extracter_class?
    @internal_hash[:show_output_from_the_extracter_class]
  end

  # ========================================================================= #
  # === is_on_slackware?
  #
  # This method will return true if we are using slackware; and false
  # otherwise.
  # ========================================================================= #
  def is_on_slackware?
    configuration?.operating_system == 'slackware'
  end

  # ========================================================================= #
  # === is_this_a_registered_binary_name?
  # ========================================================================= #
  def is_this_a_registered_binary_name?(i)
    ::RBT.has_this_binary?(i)
  end

  # ========================================================================= #
  # === configuration?
  # ========================================================================= #
  def configuration?
    RBT.configuration?
  end; alias config? configuration? # === config?

  # ========================================================================= #
  # === configure_command_to_use?
  # ========================================================================= #
  def configure_command_to_use?
    cookbook_dataset?.configure_command_to_use?
  end

  # ========================================================================= #
  # === cookbook_dataset_postinstall?
  #
  # Query from the cookbook-yaml file at hand whether we will run
  # a postinstall-step or not.
  # ========================================================================= #
  def cookbook_dataset_postinstall?(
      i = cookbook_dataset?
    )
    i and i.postinstall?
  end; alias cookbook_dataset_postinstallation? cookbook_dataset_postinstall? # === cookbook_dataset_postinstallation?

  # ========================================================================= #
  # === program_version?
  #
  # This method will return the program version in use. The default will
  # be to simply use the version stored in the cookbook dataset, but
  # we also allow for the user to supply commandline-flags to use any
  # other program version that he/she desires to use.
  # ========================================================================= #
  def program_version?
    _ = cookbook_dataset_program_version? # ← Default value.
    # ======================================================================= #
    # The user can override it.
    # ======================================================================= #
    if @internal_hash[:use_this_specific_program_version]
      _ = @internal_hash[:use_this_specific_program_version]
    end
    #_.delete('/') if _ # A bit hackish, since it is a workaround, but necessary for now (Jan 2019).
    # ^^^ the above was disabled as of September 2022. Not sure if it will be
    #     reinstated. We hopefully no longer need it now after the rewrite.
    return _
  end; alias version?        program_version? # === version?
       alias program_version program_version? # === program_version

  # ========================================================================= #
  # === can_we_continue?
  # ========================================================================= #
  def can_we_continue?
    result = @internal_hash[:can_we_continue]
    # if cookbook_dataset?.nil?
    #   initialize_dataset_for(compile_which_program?)
    # end
    # ignore_errors = @internal_hash[:ignore_errors]
    # case ignore_errors
    # # ======================================================================= #
    # # === :infer
    # # ======================================================================= #
    # when :infer
    #     ignore_errors = cookbook_dataset_ignore_errors? if cookbook_dataset?
    # end
    # if ignore_errors
    #   result = true
    # end
    return result
  end

  # ========================================================================= #
  # === appdir_prefix_points_at?
  # ========================================================================= #
  def appdir_prefix_points_at?
    real_prefix?.prefix?
  end

  # ======================================================================= #
  # === cookbook_program_name_and_program_version?
  # ======================================================================= #
  def cookbook_program_name_and_program_version?
    cookbook_dataset? and
    cookbook_dataset?.program_name_and_program_version?
  end; alias program_name_and_program_version? cookbook_program_name_and_program_version? # === program_name_and_program_version?

  # ========================================================================= #
  # === has_gir_files?
  #
  # This method will determine whether the package at hand has .gir files
  # or whether it does not.
  # ========================================================================= #
  def has_gir_files?
    !cookbook_dataset?.gir_files.empty?
  end

  # ========================================================================= #
  # === cookbook_dataset_base_dir?
  # ========================================================================= #
  def cookbook_dataset_base_dir?
    cookbook_dataset?.base_dir?
  end

  # ========================================================================= #
  # === use_this_specific_program_version?
  # ========================================================================= #
  def use_this_specific_program_version?
    @internal_hash[:use_this_specific_program_version]
  end

  # =========================================================================== #
  # === internal_pid?
  # =========================================================================== #
  def internal_pid?
    @internal_hash[:pid]
  end

  # ========================================================================= #
  # === try_to_use_the_file_windows_cookbook?
  # ========================================================================= #
  def try_to_use_the_file_windows_cookbook?
    config?.try_to_use_the_file_windows_cookbook
  end; alias try_to_use_the_file_windows_cookbook try_to_use_the_file_windows_cookbook? # === try_to_use_the_file_windows_cookbook

  # ========================================================================= #
  # === install_only_headers?
  # ========================================================================= #
  def install_only_headers?
    @internal_hash[:install_only_the_headers]
  end; alias install_only_the_headers? install_only_headers? # === install_only_the_headers?

  # ========================================================================= #
  # === cookbook_dataset_meson_configure_options?
  #
  # This method will return all options used for the "meson" build
  # system of the program at hand, should meson be used at all.
  # ========================================================================= #
  def cookbook_dataset_meson_configure_options?
    cookbook_dataset?.meson_configure_options?
  end; alias meson_configure_options?          cookbook_dataset_meson_configure_options? # === meson_configure_options?
       alias cookbook_meson_configure_options? cookbook_dataset_meson_configure_options? # === cookbook_meson_configure_options?

  # ========================================================================= #
  # === cmake_configure_options?
  # ========================================================================= #
  def cmake_configure_options?
    cookbook_dataset?.cmake_configure_options?
  end

  # ========================================================================= #
  # === make_use_of_the_cookbook_aliases?
  # ========================================================================= #
  def make_use_of_the_cookbook_aliases?
    @internal_hash[:make_use_of_the_cookbook_aliases]
  end

  # ========================================================================= #
  # === missing_cookbooks_dataset?
  # ========================================================================= #
  def missing_cookbooks_dataset?
    !a_program_has_been_initialized?
  end

  # ========================================================================= #
  # === base_dir?
  # ========================================================================= #
  def base_dir?
    cookbook_for?.base_dir?.to_s
  end

  # ========================================================================= #
  # === a_program_has_been_initialized?
  # ========================================================================= #
  def a_program_has_been_initialized?
    !main_dataset?.nil?
  end

  # ========================================================================= #
  # === exit_before_make?
  # ========================================================================= #
  def exit_before_make?
    @internal_hash[:exit_before_make]
  end

  # ========================================================================= #
  # === cookbook_dataset_ignore_errors?
  # ========================================================================= #
  def cookbook_dataset_ignore_errors?
    cookbook_dataset? and cookbook_dataset?.ignore_errors?
  end

  # ========================================================================= #
  # === tags?
  #
  # Note that this method will attempt to return an Array.
  # ========================================================================= #
  def tags?
    cookbook_dataset?.tags?
  end; alias tags tags? # === tags

  # ========================================================================= #
  # === cookbooks_binaries?
  # ========================================================================= #
  def cookbooks_binaries?
    cookbooks_dataset?.binaries?
  end; alias binaries? cookbooks_binaries? # === binaries?

  # ========================================================================= #
  # === use_meson_configure_options?
  # ========================================================================= #
  def use_meson_configure_options?
    @internal_hash[:use_meson_configure_options]
  end

  # ========================================================================= #
  # === symlink_headers?
  #
  # Use this method if you need to find out whether we shall symlink
  # the .h files into the "/usr/include/" hierarchy or whether we
  # should not.
  # ========================================================================= #
  def symlink_headers?
    _ = cookbook_dataset_symlink_headers?
    unless @internal_hash[:symlink_headers].nil?
      _ = @internal_hash[:symlink_headers]
    end
    _
  end; alias symlink_headers_into_usr_include_hierarchy? symlink_headers? # === symlink_headers_into_usr_include_hierarchy?

  # ========================================================================= #
  # === mode?
  # ========================================================================= #
  def mode?
    cookbook_dataset?.mode?
  end

  # ========================================================================= #
  # === hash?
  # ========================================================================= #
  def hash?
    cookbook_dataset?.hash?
  end

  # ========================================================================= #
  # === remove_the_extracted_archive?
  # ========================================================================= #
  def remove_the_extracted_archive?
    !keep_the_extracted_archive?
  end

  # ========================================================================= #
  # === pid?
  # ========================================================================= #
  def pid?
    $?.pid
  end

  # ========================================================================= #
  # === continue_after_make_step?
  # ========================================================================= #
  def continue_after_make_step?
    @internal_hash[:continue_after_make_step]
  end

  # ========================================================================= #
  # === compile_this_program?
  # ========================================================================= #
  def compile_this_program?
    @internal_hash[:compile_this_program]
  end; alias program_name?           compile_this_program? # === program_name?
       alias compile_which_program?  compile_this_program? # === compile_which_program?
       alias install_which_programs? compile_this_program? # === install_which_programs?

end; end; end
