#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::Statistics::DisplayBarplotStatistics
#
# Usage example:
#
#   RBT::Action::Statistics::DisplayBarplotStatistics.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/statistics/display_barplot_statistics.rb'
# RBT::Action::Statistics::DisplayBarplotStatistics.new(ARGV)
# RBT.display_barplot_statistics
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

module Statistics

class DisplayBarplotStatistics < RBT::Action # === RBT::Action::Statistics::DisplayBarplotStatistics

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    array = RBT.statistics?
    require 'unicode_plot'
    UnicodePlot.barplot(
      data: Hash[array], # {'foo': 20, 'bar': 50},
      title: 'RBT - increase in the number of programs over the years' 
    ).render
  end

  # ========================================================================= #
  # === RBT::Action::Statistics::DisplayBarplotStatistics[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

# =========================================================================== #
# === RBT.display_barplot_statistics
# =========================================================================== #
def self.display_barplot_statistics
  RBT::Action::Statistics::DisplayBarplotStatistics.new
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.display_barplot_statistics
end # displaybarplotstatistics