#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::Statistics::ShowAllStatistics
#
# This actionable class can be used to output anything that may be related
# to statistics, within the RBT project/namespace.
#
# Usage example:
#
#   RBT::Action::Statistics::ShowAllStatistics.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/statistics/show_all_statistics.rb'
# RBT::Action::Statistics::ShowAllStatistics.new(ARGV)
# RBT.show_all_statistics
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

module Statistics

class ShowAllStatistics < RBT::Action # === RBT::Action::Statistics::ShowAllStatistics

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    cliner
    action(:compile_strategies)
    cliner
    action(:show_how_many_files_are_tracked)
    cliner
    action(:show_compile_time_statistics)
    cliner
  end

  # ========================================================================= #
  # === RBT::Action::Statistics::ShowAllStatistics[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

# =========================================================================== #
# === RBT.show_all_statistics
# =========================================================================== #
def self.show_all_statistics
  RBT::Action::Statistics::ShowAllStatistics.new
end; self.instance_eval { alias show_statistics show_all_statistics } # === RBT.show_statistics

end

if __FILE__ == $PROGRAM_NAME
  RBT.show_all_statistics
end # showallstatistics