#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Actions::MetaActions
#
# MetaActions combine different activities into one "larger" activity, such
# as "the user wants to install a linux-from-scratch system".
#
# Note that this file will pull in the file rbt/actions/actions.rb.
# =========================================================================== #
module RBT

module Actions

module MetaActions # === RBT::Actions::MetaActions

  require 'rbt/actions/actions.rb'

  # ========================================================================= #
  # === RBT::Actions::MetaActions.install_linux_from_scratch
  #
  # Note that entries in the following Array starting with a ':' character,
  # will refer to a special, "actionable action".
  # ========================================================================= #
  def self.install_linux_from_scratch
    # ======================================================================= #
    # Build up an array of programs to be installed next:
    # ======================================================================= #
    array_install_these_programs = %w(
      manpages
      ianaetc
      glibc
      zlib
      bzip2
      xz
      zstd
      file
      readline
      m4
      bc
      flex
      tcl
      expect
      dejagnu
      pkgconf
      binutils
      gmp
      mpfr
      mpc
      attr
      acl
      libcap
      libxcrypt
      shadow
      gcc
      ncurses
      sed
      psmisc
      gettext
      bison
      grep
      bash
      libtool
      gdbm
      gperf
      expat
      inetutils
      less
      perl
      xmlparser
      intltool
      autoconf
      automake
      openssl
      kmod
      libelf
      libffi
      python
      flitcore
      wheel
      setuptools
      ninja
      meson
      coreutils
      check
      diffutils
      gawk
      findutils
      groff
      grub
      gzip
      iproute2
      kbd
      libpipeline
      make
      patch
      tar
      texinfo
      ruby
      vim
      markupsafe
      jinja2
      systemd_udev
      mandb
      procpsng
      utillinux
      e2fsprogs
      sysklogd
      sysvinit
    )
    array_install_these_programs.each {|this_program|
      acompile(this_program) # This is "action compile".
    }
  end; self.instance_eval { alias lfs install_linux_from_scratch } # === RBT::Actions::MetaActions.lfs

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Actions::MetaActions.install_linux_from_scratch  
end