#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::AllowedCookbookEntries
#
# The purpose of this class is to simply show to the user which cookbook
# entries are legal/valid and which ones are not.
#
# You can query this information from the commandline as well, by
# issuing:
#
#   cookbooks --allowed-cookbook-entries
#
# =========================================================================== #
# require 'rbt/checks_and_validations/allowed_cookbook_entries.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class AllowedCookbookEntries < Base # === RBT::Cookbooks::AllowedCookbookEntries

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === RBT::Cookbooks::AllowedCookbookEntries.e
  # ========================================================================= #
  def self.e(i = '')
    RBT.e(i)
  end

  # ========================================================================= #
  # === RBT::Cookbooks::AllowedCookbookEntries.rev
  # ========================================================================= #
  def self.rev
    ::RBT.rev
  end

  # ========================================================================= #
  # === RBT::Cookbooks::AllowedCookbookEntries.report_allowed_entries
  # ========================================================================= #
  def self.report_allowed_entries(
      namespace = inspect
    )
    e; opnn(namespace)
    e
    e
    e rev+'The following entries are registered in the file '
    e ::RBT.sfile('registered_cookbook_entries.rb')+rev+', in the '
    e rev+'constant '+
      ::RBT.simp('REGISTERED_COOKBOOK_ENTRIES')+
      rev+', and thus valid:'
    e
    REGISTERED_COOKBOOK_ENTRIES.each_with_index {|entry, index|
      index += 1
      colourized_and_padded_index = ::RBT.sfancy(index.to_s.rjust(3))
      entry = Colours.mediumaquamarine(entry)
      e "#{colourized_and_padded_index}#{rev}) #{entry}"
    }; e # Need to output a terminating-newline here for consistency with the above.
  end

  # ========================================================================= #
  # === run
  #
  # We redirect to a class method here.
  # ========================================================================= #
  def run
    AllowedCookbookEntries.report_allowed_entries # Call the class method here.
  end

  # ========================================================================= #
  # === AllowedCookbookEntries.opnn
  # ========================================================================= #
  def self.opnn(
      namespace = NAMESPACE
    )
    Opn.opn(
      use_this_as_namespace: namespace
    )
  end

  # ========================================================================= #
  # === RBT::Cookbooks::AllowedCookbookEntries[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::AllowedCookbookEntries.new
end # acookbooks