#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::ReportMissingHomepageEntries
#
# This small class will simply report which .yml file lacks a homepage
# entry, or whether that homepage entry is empty.
#
# Erroneous entries will be reported to the user in such an event.
# =========================================================================== #
# require 'rbt/checks_and_validations/report_missing_homepage_entries.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class ReportMissingHomepageEntries < RBT::Base # === RBT::Cookbooks::ReportMissingHomepageEntries

  require 'rbt/aliases/aliases.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    case run_already
    when :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    e
    e 'Currently no help options are documented for this class.'
    e
  end

  # ========================================================================= #
  # === menu
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === missing_homepage --help
      # ===================================================================== #
      when /^-?-?help$/i
        show_help
        exit
      end
    end
  end

  # ========================================================================= #
  # === report_this_program_has_a_missing_homepage_entry
  # ========================================================================= #
  def report_this_program_has_a_missing_homepage_entry(i)
    e "  #{orange(i)} #{rev}has no #{steelblue('homepage')} #{rev}entry."
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    all_available_programs?.each {|this_program|
      _ = nil
      if does_this_expanded_cookbook_file_exist_for_this_program? this_program
        dataset = load_dataset_from_this_expanded_cookbook(this_program)
      else # else load from the default cookbook directory.
        dataset = load_yaml_file_from_the_cookbook_directory_for_this_program(this_program)
      end
      if dataset.has_key? this_program
        dataset = dataset[this_program]
      end
      the_homepage = dataset['homepage'.to_sym]
      if the_homepage and the_homepage.empty?
        report_this_program_has_no_homepage_entry(this_program)
      else
        if dataset.has_key? 'homepage'.to_sym
          if the_homepage == 'unknown'
            report_this_program_has_no_complete_homepage_entry(this_program)
          end
        else
          report_this_program_has_a_missing_homepage_entry(this_program)
        end
      end
    }
  end

  # ========================================================================= #
  # === report_this_program_has_no_complete_homepage_entry
  # ========================================================================= #
  def report_this_program_has_no_complete_homepage_entry(i)
    e steelblue(i.ljust(30))+
      rev+' has an incomplete/missing homepage entry.'
  end
  
  # ========================================================================= #
  # === report_this_program_has_no_homepage_entry
  # ========================================================================= #
  def report_this_program_has_no_homepage_entry(i)
    e steelblue(i.ljust(30))+
      rev+' has no homepage entry.'
  end
end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::ReportMissingHomepageEntries.new(ARGV)
end # missinghomepage