#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CompileViaSQLDatabase
# =========================================================================== #
# require 'rbt/compile_via_sql_database/compile_via_sql_database.rb'
# RBT::CompileViaSQLDatabase.new
# =========================================================================== #
require 'rbt/compile_base/compile_base.rb'

module RBT

class CompileViaSQLDatabase < ::RBT::CompileBase # === RBT::CompileViaSQLDatabase

  require 'rbt/prefix/prefix.rb'

  # ========================================================================= #
  # === DEFAULT_LOCATION_OF_THE_DATABASE
  # ========================================================================= #
  DEFAULT_LOCATION_OF_THE_DATABASE =
    ::RBT.log_dir?+'database/registered_programs.db'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    map_the_first_argument_onto_the_compile_this_program_variable
    case run_already
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    try_to_require_the_extracter_gem
    # ======================================================================= #
    # === :location_of_the_database
    # ======================================================================= #
    @internal_hash[:location_of_the_database] =
      DEFAULT_LOCATION_OF_THE_DATABASE
    # ======================================================================= #
    # === :use_this_prefix
    # ======================================================================= #
    @internal_hash[:use_this_prefix] = RBT::Prefix.new('/usr/')
    # ======================================================================= #
    # === :local_path
    # ======================================================================= #
    set_local_path(nil)
    # ======================================================================= #
    # === :archive_type
    # ======================================================================= #
    @internal_hash[:archive_type] = nil
  end

  # ========================================================================= #
  # === set_local_path
  # ========================================================================= #
  def set_local_path(i)
    @internal_hash[:local_path] = i
  end

  # ========================================================================= #
  # === internal_hash_archive_type?
  # ========================================================================= #
  def internal_hash_archive_type?
    @internal_hash[:archive_type]
  end

  # ========================================================================= #
  # === set_prefix
  # ========================================================================= #
  def set_prefix(i)
    case i
    # ======================================================================= #
    # === :appdir_prefix
    # ======================================================================= #
    when :appdir_prefix
      result = `sqlite3 #{location_of_the_database?} "select * from REGISTERED_PROGRAMS where PROGRAM_NAME = '#{compile_which_program?}';"`.split('|')[8]
      i = return_appdir_prefix_for(
            result
          )
    end
    i = i.to_s
    i << '/' unless i.end_with? '/'
    @internal_hash[:use_this_prefix].set_new_prefix(i)
  end; alias set_new_prefix set_prefix # === set_new_prefix

  # ========================================================================= #
  # === location_of_the_database?
  # ========================================================================= #
  def location_of_the_database?
    @internal_hash[:location_of_the_database]
  end; alias database_location? location_of_the_database? # === database_location?

  # ========================================================================= #
  # === set_compile_this_program
  # ========================================================================= #
  def set_compile_this_program(
      i = :random
    )
    case i
    # ======================================================================= #
    # === :random
    #
    # In this case we query directly from the sqlite-database.
    # ======================================================================= #
    when :random
      i = return_a_random_program_from_the_sqlite_database
    end
    @internal_hash[:compile_this_program] = i.to_s
  end

  # ========================================================================= #
  # === n_programs_are_available?
  # ========================================================================= #
  def n_programs_are_available?
    available_programs?.size
  end; alias n_programs_available?    n_programs_are_available? # === n_programs_available?
       alias n_programs_are_available n_programs_are_available? # === n_programs_are_available

  # ========================================================================= #
  # === is_it_a_rubygem?                                            (gem tag)
  # ========================================================================= #
  def is_it_a_rubygem?(
      i = internal_hash_archive_type?
    )
    i.end_with?('.gem')
  end

  # ========================================================================= #
  # === local_path?
  # ========================================================================= #
  def local_path?
    @internal_hash[:local_path]
  end

  # ========================================================================= #
  # === compile_which_program?
  # ========================================================================= #
  def compile_which_program?
    @internal_hash[:compile_this_program].to_s
  end; alias program_name_that_is_to_be_installed? compile_which_program? # === program_name_that_is_to_be_installed?

  # ========================================================================= #
  # === do_install_via_meson_then_use_ninja
  # ========================================================================= #
  def do_install_via_meson_then_use_ninja
    esystem "meson --prefix=#{use_which_prefix?} --libdir=#{use_which_prefix?}lib BUILD_DIRECTORY"
    cd 'BUILD_DIRECTORY'
    esystem 'ninja'
    esystem 'ninja install'
  end

  # ========================================================================= #
  # === do_install_via_configure_then_make_then_make_install
  # ========================================================================= #
  def do_install_via_configure_then_make_then_make_install
    esystem "./configure --prefix=#{use_which_prefix?}"
    run_make_then_make_install
  end

  # ========================================================================= #
  # === run_make_then_make_install
  # ========================================================================= #
  def run_make_then_make_install
    run_make
    run_make_install
  end

  # ========================================================================= #
  # === use_which_prefix?
  # ========================================================================= #
  def use_which_prefix?
    @internal_hash[:use_this_prefix].to_s
  end; alias prefix? use_which_prefix? # === prefix?

  # ========================================================================= #
  # === use_this_prefix?
  # ========================================================================= #
  def use_this_prefix?
    @internal_hash[:use_this_prefix]
  end; alias use_this_prefix_object? use_this_prefix? # === use_this_prefix_object?

  # ========================================================================= #
  # === do_install_via_cmake                                      (cmake tag)
  # ========================================================================= #
  def do_install_via_cmake
    esystem "cmake -DCMAKE_INSTALL_PREFIX=#{use_which_prefix?} ."
    run_make_then_make_install
  end

  # ========================================================================= #
  # === run_make_install
  # ========================================================================= #
  def run_make_install
    esystem 'make install'
  end

  # ========================================================================= #
  # === run_make
  # ========================================================================= #
  def run_make
    esystem 'make'
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  #
  # Invocation examples:
  #
  #   sqler --random-program
  #   sqler RANDOM
  #   sqler random
  #
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i and i.is_a?(Array)
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === sqler kldap --ntrad
      # ===================================================================== # 
      when /^-?-?ntrad/i
        set_new_prefix(:appdir_prefix)
      # ===================================================================== #
      # === sqler --random
      # ===================================================================== #
      when /^-?-?RANDOM/i
        set_compile_this_program(:random)
      end
    end
  end

  # ========================================================================= #
  # === assumed_local_file_path_for?
  #
  # This method was specifically added for the Libui GUI wrapper. It is
  # normally not in use by this class, if used on the commandline.
  # ========================================================================= #
  def assumed_local_file_path_for?(this_program)
    if this_program and is_this_program_included_in_the_database?(this_program)
      result = `sqlite3 #{location_of_the_database?} "select * from REGISTERED_PROGRAMS where PROGRAM_NAME = '#{this_program}';"`
      splitted = result.strip.split('|')
      return splitted[2] # This is the local path - at the least on 27.08.2022.
    end
    return this_program
  end

  # ========================================================================= #
  # === available_programs?
  # ========================================================================= #
  def available_programs?
    result = `sqlite3 #{location_of_the_database?} "select * from REGISTERED_PROGRAMS;"`.split("\n")
    array = result.map {|entry|
      splitted = entry.split('|')
      splitted[1]
    }
    return array
  end

  # ========================================================================= #
  # === return_a_random_program_from_the_sqlite_database         (random tag)
  # ========================================================================= #
  def return_a_random_program_from_the_sqlite_database
    cmd = "SELECT * FROM REGISTERED_PROGRAMS ORDER BY RANDOM() LIMIT 1;"
    result = `sqlite3 #{location_of_the_database?} "#{cmd}"`
    if result and result.include? '|'
      result = result.split('|')[1]
    end
    return result
  end

  # ========================================================================= #
  # === assumed_version_of_this_program?
  #
  # This method will use a SQL query to determine the (assumed) version
  # of a particular program. It was added specifically because the
  # LibUI GUI may need to know this information.
  # ========================================================================= #
  def assumed_version_of_this_program?(this_program)
    this_program = this_program.dup
    this_program.delete!('-') if this_program.include?('-')
    result = `sqlite3 #{location_of_the_database?} "select * from REGISTERED_PROGRAMS where PROGRAM_NAME = '#{this_program}';"`
    if result and result.include? '|'
      result = result.split('|')[4] # Version is at position 4.
      # So we use the local name for that.
      result = ProgramInformation.return_version(File.basename(result))
    end
    return result
  end

  # ========================================================================= #
  # === do_extract_this_locally_existing_archive
  # ========================================================================= #
  def do_extract_this_locally_existing_archive(
      what,
      to = '/tmp/'
    )
    _result_containing_the_extracter_class = Extracter.what_to(what, to)
  end; alias extract_the_program do_extract_this_locally_existing_archive # === extract_the_program

  # ========================================================================= #
  # === is_this_program_included_in_the_database?
  # ========================================================================= #
  def is_this_program_included_in_the_database?(i)
    result = `sqlite3 #{location_of_the_database?} "select * from REGISTERED_PROGRAMS where PROGRAM_NAME = '#{i}';"`
    result and result.include?('|') and (result.split('|')[1] == i)
  end; alias is_this_program_included? is_this_program_included_in_the_database? # === is_this_program_included?

  # ========================================================================= #
  # === RBT::CompileViaSQLDatabase.location_of_the_database?
  # ========================================================================= #
  def self.location_of_the_database?
    DEFAULT_LOCATION_OF_THE_DATABASE
  end

  # ========================================================================= #
  # === do_the_actual_compilation
  # ========================================================================= #
  def do_the_actual_compilation(
      _ = location_of_the_database?
    )
    cd(log_directory?) # We must always try to cd to the base directory.
    if File.exist? _
      e rev+'Loading the sqlite database from '+sfile(_)+' next.'
      e steelblue(n_programs_are_available?.to_s)+
        ' programs are available in that sqlite-database.'
      _ = compile_which_program?
      if _ and is_this_program_included_in_the_database?(_)
        e 'The program '+sfancy(_)+' is registered. Thus, proceeding to '\
          'compile / install it.'
        result = `sqlite3 #{location_of_the_database?} "select * from REGISTERED_PROGRAMS where PROGRAM_NAME = '#{_}';"`
        # =================================================================== #
        # Let's keep a reminder for how the splitted dataset may look like:
        #
        #    2604, php, /home/x/src/php/php-8.1.9.tar.xz, 8.1.9,
        #    /home/Temp/rbt/php-8.1.9/, configure, 1, unknown,
        #    php-8.1.9, https://www.php.net/, .tar.xz, 11787892
        #
        # =================================================================== #
        splitted = result.strip.split('|')
        local_path       = splitted[2] # is at 2.
        set_local_path(local_path) # Keep a reference, for later use.
        original_extract_where_to = splitted[4].dup # is at 4.
        compile_how      = splitted[5] # is at 5.
        archive_type     = splitted[10] # is at 10.
        @internal_hash[:archive_type] = archive_type
        @internal_hash[:program_name_and_program_version] = splitted[8]
        extract_where_to = rds(
                             File.dirname(original_extract_where_to)+'/'
                           )
        e 'The assumed local path is:'
        e
        e sfancy("  #{local_path}")
        e
        # =================================================================== #
        # Check whether the file exists locally.
        # =================================================================== #
        if File.exist?(local_path)
          e 'The file exists locally. Proceeding to compile/install it.'
          do_extract_this_locally_existing_archive(
            local_path,
            extract_where_to
          ) unless is_it_a_rubygem?(local_path)
          cd original_extract_where_to
          do_compile_via_this_variant(compile_how)
          unless are_we_on_windows?
            handle_postinstallation_symlink
          end
        else
           e 'The file does not exist locally. Trying '\
             'to download it next.'
        end
      else
        e 'The program '+steelblue(_)+' is NOT included. Please provide'
        e 'a registered program.'
      end
    else
      no_file_exists_at(_)
    end
  end

  # ========================================================================= #
  # === makes_use_of_an_appdir_prefix?
  # ========================================================================= #
  def makes_use_of_an_appdir_prefix?
    use_this_prefix_object?.makes_use_of_an_appdir_prefix? # Tap into the prefix-object.
  end

  # ========================================================================= #
  # === do_compile_via_this_variant
  # ========================================================================= #
  def do_compile_via_this_variant(i)
    if is_it_a_rubygem?
      RBT.action(:install_this_rubygem, local_path?) # Delegate to the specific action here.
      return
    end
    case i
    # ======================================================================= #
    # === configure
    # ======================================================================= #
    when 'configure'
      do_install_via_configure_then_make_then_make_install
    # ======================================================================= #
    # === meson
    # ======================================================================= #
    when 'meson'
      do_install_via_meson_then_use_ninja
    # ======================================================================= #
    # === cmake
    # ======================================================================= #
    when 'cmake'
      do_install_via_cmake
    else
      e
      opne tomato("Unhandled variant: #{i}")
      e
      opne 'Please register this compilation-mode or handle otherwise.'
    end
  end

  # ========================================================================= #
  # === program_name_and_program_version?
  # ========================================================================= #
  def program_name_and_program_version?
    @internal_hash[:program_name_and_program_version]
  end

  # ========================================================================= #
  # === map_the_first_argument_onto_the_compile_this_program_variable
  #
  # This is normally called before menu().
  # ========================================================================= #
  def map_the_first_argument_onto_the_compile_this_program_variable(
      i = first_argument?
    ) 
    set_compile_this_program(i)
  end

  # ========================================================================= #
  # === handle_postinstallation_symlink
  # ========================================================================= #
  def handle_postinstallation_symlink
    if makes_use_of_an_appdir_prefix?
      action(:to_current, prefix?.to_s)
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    menu
    do_the_actual_compilation
  end

end; end

if __FILE__ == $PROGRAM_NAME
  _object = RBT::CompileViaSQLDatabase.new(ARGV)
  # pp _object.available_programs?
end # sqlcompile php