#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/constants/predefined_installation_instructions.rb'
# RBT::FILE_SETUP_PY
# =========================================================================== #
module RBT

  require 'rbt/project/project.rb'

  # ========================================================================= #
  # === HASH_INSTALLATION_INSTRUCTIONS
  #
  # This Hash gives us an easier handle to sanitize our data.
  #
  # It has some constant-shortcuts that are convenient for use lateron.
  #
  # For instance, RINSTALL means "ruby install", PINSTALL means
  # "python install".
  # ========================================================================= #
  HASH_INSTALLATION_INSTRUCTIONS = {}

  # ========================================================================= #
  # === RBT.file_predefined_installation_instructions
  #
  # Method to return where the file location is. There used to be
  # a constant available called FILE_PREDEFINED_INSTALLATION_INSTRUCTIONS
  # but this was removed in January 2019.
  # ========================================================================= #
  def self.file_predefined_installation_instructions(
      use_this_directory = 
        RBT.project_yaml_directory?
    ) 
    "#{use_this_directory}"\
    "predefined_installation_instructions/predefined_installation_instructions.yml"
  end

  # ========================================================================= #
  # === PREDEFINED_INSTALLATION_INSTRUCTIONS
  #
  # Load up the dataset for the predefined installation instructions next.
  #
  # This will ultimately yield a Hash.
  # ========================================================================= #
  _ = file_predefined_installation_instructions
  if File.exist? _
    require 'rbt/requires/require_yaml.rb' # We need support for YAML past this point.
    @predefined_installation_instructions = YAML.load_file(_)
    @predefined_installation_instructions.each_pair { |key, value|
      HASH_INSTALLATION_INSTRUCTIONS[key] = value
    }
  else
    @predefined_installation_instructions = {} # Define an empty Hash if the file does not exist.
  end

  # ========================================================================= #
  # === RBT.predefined_installation_instructions?
  # ========================================================================= #
  def self.predefined_installation_instructions?
    @predefined_installation_instructions
  end; self.instance_eval { alias pii? predefined_installation_instructions? } # === RBT.pii?

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e 'The file can be found at:'
  e
  e "  #{RBT.file_predefined_installation_instructions}"
  e
  pp RBT.predefined_installation_instructions?
  e
  pp RBT::HASH_INSTALLATION_INSTRUCTIONS
  e
end