#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CopyTheIdealCookbook
#
# The purpose of this class is to copy the ideal cookbook. This may seem
# a little bit overkill to create a dedicated class for this, but I wanted
# to change my own habit of constantly copying old .yml files rather than
# use a new, empty cookbook. Plus, we can also add more features and
# functionality into this over time, so this is another reason why that
# was done.
# =========================================================================== #
# require 'rbt/cookbooks/copy_the_ideal_cookbook/copy_the_ideal_cookbook.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class CopyTheIdealCookbook < Base # === RBT::Cookbooks::CopyTheIdealCookbook

  # ========================================================================= #
  # === SPLIT_AT_THIS_TOKEN
  # ========================================================================= #
  SPLIT_AT_THIS_TOKEN = '-------------------------------------------------------------------------------'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset 
    set_commandline_arguments(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === :file_the_ideal_cookbook
    # ======================================================================= #
    @internal_hash[:file_the_ideal_cookbook] =
      '/home/x/programming/ruby/src/rbt/doc/the_ideal_cookbook/the_ideal_cookbook.md' 
  end; alias clear reset # === clear

  # ========================================================================= #
  # === file_the_ideal_cookbook?
  # ========================================================================= #
  def file_the_ideal_cookbook?
    @internal_hash[:file_the_ideal_cookbook]
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = file_the_ideal_cookbook?
    new_target = return_pwd+File.basename(_)
    dataset = file_read(_)
    if dataset.include? SPLIT_AT_THIS_TOKEN
      dataset = dataset.split(SPLIT_AT_THIS_TOKEN).last
    end
    dataset = dataset.dup if dataset.frozen?
    first = first?
    if first and !first.empty?
      dataset.sub!(
        /^program_name_goes_in_here/,
        first
      )
      new_target = return_pwd+File.basename(first)+'.yml'
      if dataset.include? 'some_website'
        dataset.gsub!(/some_website/,first)
      end
    end
    opne "#{rev}Now generating a new cookbook file, into "\
         "the current working directory"
    opne "at `#{sfile(new_target)}`."
    # ======================================================================= #
    # Fix the last_update entry next:
    # ======================================================================= #
    if dataset.include? 'last_update:'
      dataset.sub!(
        /^ last_update: (.+)/,
        ' last_update: '+
        convert_dd_mm_yyyy_to_its_long_variant(dd_mm_yyyy.to_s)
      )
    end
    dataset.lstrip!
    write_what_into(dataset, new_target)
    @absolute_path = File.absolute_path(new_target) # And keep a reference to the absolute path as well here.
  end

  # ========================================================================= #
  # === report_the_path
  # ========================================================================= #
  def report_the_path(i = path?)
    e sfile(i)
  end

  # ========================================================================= #
  # === path?
  # ========================================================================= #
  def path?
    @absolute_path
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::CopyTheIdealCookbook.new(ARGV)
end # copytheidealcookbook htop