#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CreateGlobalYamlDatabase
#
# This class will fetch the information from ALL the individual
# .yml files and create a giant .yml file with all the
# information from that.
# =========================================================================== #
# require 'rbt/generate_files/create_global_yaml_database.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class CreateGlobalYamlDatabase < Base # === RBT::Cookbooks::CreateGlobalYamlDatabase

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === STORE_HERE
  # ========================================================================= #
  STORE_HERE =
    "#{RBT.rbt_log_directory?}global_yaml_database.yml"
  
  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @dataset
    # ======================================================================= #
    @dataset = Hash.new # This is the main hash of this class.
  end

  # ========================================================================= #
  # === CreateGlobalYamlDatabase.store_where?
  # ========================================================================= #
  def self.store_where?
    STORE_HERE
  end

  # ========================================================================= #
  # === store_into_yaml_file
  # ========================================================================= #
  def store_into_yaml_file
    into = STORE_HERE
    opne "Now storing into `#{sfile(into)}`."
    if File.exist? into
      File.delete(into) # Always get rid of the old yaml file first.
    end
    what = YAML.dump(@dataset)
    save_what_into(what, into)
  end

  # ========================================================================= #
  # === obtain_all_programs
  # ========================================================================= #
  def obtain_all_programs
    @all_programs = available_programs?
  end

  # ========================================================================= #
  # === iterate_over_every_program
  # ========================================================================= #
  def iterate_over_every_program
    @all_programs.each_with_index {|program, index|
      dataset = action(:SanitizeCookbook, program)
      dataset = dataset.dataset?
      hash = {}
      hash[program] = dataset
      _ = {}
      dataset.each_pair {|key, value|
        if key
          _[key] = value
        end
      }
      dataset = _
      opne "#{rev}Now working on the program `#{royalblue(program)}#{rev}`."
      @dataset.update(hash)
    }
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    obtain_all_programs
    iterate_over_every_program
    store_into_yaml_file
  end

end; end

# =========================================================================== #
# === RBT.generate_global_yaml_database
# =========================================================================== #
def self.generate_global_yaml_database
  RBT::Cookbooks::CreateGlobalYamlDatabase.new
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::CreateGlobalYamlDatabase.new
end # globaldatabase