#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::GeneratePdfTutorial
#
# This class will generate a .pdf tutorial, similar to the way
# how we do this for the DiamondShell project.
# =========================================================================== #
# require 'rbt/generate_files/generate_pdf_tutorial.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class GeneratePdfTutorial < Base

  require 'open-uri'

  # ========================================================================= #
  # === URL
  #
  # This URl specifies which URL is to be used.
  # ========================================================================= #
  URL =
    'http://localhost/programming/ruby/src/rbt/doc/www/RBT.cgi?pdf'

  # ========================================================================= #
  # === STORE_PDF_FILE_HERE
  #
  # Where to store the .pdf file.
  # ========================================================================= #
  STORE_PDF_FILE_HERE =
    '/home/Temp/lighty/RBT.pdf'

  # ========================================================================= #
  # === RBT::GeneratePdfTutorial.store_pdf_file_where?
  # ========================================================================= #
  def self.store_pdf_file_where?
    STORE_PDF_FILE_HERE
  end

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    @store_where = STORE_PDF_FILE_HERE
  end

  # ========================================================================= #
  # === store_where?
  # ========================================================================= #
  def store_where?
    @store_where
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    # ======================================================================= #
    # First, always get rid of the old .pdf file.
    # ======================================================================= #
    remove_file(store_where?) if File.exist?(store_where?)
    # ======================================================================= #
    # Here we should have generated it.
    # ======================================================================= #
    opne 'Next trying the URL `'+sfancy(URL)+'`.'
    open(URL)
    result = "#{rev}The .pdf file should now exist at "\
             "`#{sfile(store_where?)}#{rev}`".dup
    if File.exist? store_where?
      result << ' (Yes, it does exist there)'
    end
    result << '.'
    opne result 
  end

end

# =========================================================================== #
# === RBT.generate_pdf_tutorial
# =========================================================================== #
def self.generate_pdf_tutorial
  RBT::GeneratePdfTutorial.new
end

# =========================================================================== #
# === RBT.generate_pdf_tutorial_then_upload_it
#
# Use this method to upload the .pdf tutorial to a remote host.
# =========================================================================== #
def self.generate_pdf_tutorial_then_upload_it
  RBT.generate_pdf_tutorial
  # Next comes the upload part.
  require 'ftp_paradise/without_gui_bindings'
  connection = FtpParadise.connect to: :shevy
  connection.set_to_binary_mode
  connection.upload_this_file RBT::GeneratePdfTutorial.store_pdf_file_where?
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::GeneratePdfTutorial.new
end # generatepdftutorial