#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::GUI::UniversalWidgets::RecipeMaker
#
# This class shall provide a ruby-gtk3 interface to the create-a-cookbook
# functionality of the RBT project.
#
# Usage example:
#
#   RBT::GUI::UniversalWidgets::RecipeMaker.new(ARGV)
#
# =========================================================================== #
# require 'rbt/gui/universal_widgets/recipe_maker/recipe_maker.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module GUI

module UniversalWidgets

class RecipeMaker < ::RBT::Base # === RBT::GUI::UniversalWidgets::RecipeMaker

  require 'universal_widgets/base_module/base_module.rb'
  include ::UniversalWidgets::BaseModule

  require 'rbt/toplevel_methods/author_name.rb'
  require 'rbt/toplevel_methods/toplevel_methods.rb'
  require 'rbt/toplevel_methods/files_and_directories_related_methods.rb'
  require 'rbt/cookbooks/create_cookbook_yaml_file/create_cookbook_yaml_file.rb'
  require 'rbt/requires/require_the_cookbook_class.rb'
  require 'rbt/cookbooks/url/url.rb'

  begin
    require 'cliner/module'
  rescue LoadError; end

  begin
    require 'wget'
  rescue LoadError; end

  # ========================================================================= #
  # === USE_THESE_CSS_RULES
  # ========================================================================= #
  USE_THESE_CSS_RULES = '
  '

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Recipe Maker'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = '82% or minimum 1280px'

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = '40% or minimum 300px'

  # ========================================================================= #
  # === RBT::GUI::Gtk::RecipeMaker.height?
  # ========================================================================= #
  def self.height?
    HEIGHT
  end

  # ========================================================================= #
  # === RBT::GUI::Gtk::RecipeMaker.width?
  # ========================================================================= #
  def self.width?
    WIDTH
  end

  # ========================================================================= #
  # === USE_THIS_FONT
  # ========================================================================= #
  USE_THIS_FONT = :dejavu_condensed_20 # or :hack_20

  # ========================================================================= #
  # === LARGER_FONT
  # ========================================================================= #
  LARGER_FONT   = :dejavu_condensed_22

  # ========================================================================= #
  # === SMALLER_FONT
  # ========================================================================= #
  SMALLER_FONT  = :dejavu_condensed_18

  # ========================================================================= #
  # === GUESS_NAME
  # ========================================================================= #
  GUESS_NAME = true              # if true, we attempt to guess the name

  # ========================================================================= #
  # === DEBUG
  # ========================================================================= #
  DEBUG = false                  # set this to false to recieve no debug info

  # ========================================================================= #
  # === VERSION_RECIPE_MAKER
  # ========================================================================= #
  VERSION_RECIPE_MAKER = '1.0.7' # which version do we have?

  # ========================================================================= #
  # === USE_OVERRIDE
  # ========================================================================= #
  USE_OVERRIDE = false           # if this is true, we set the override var

  # ========================================================================= #
  # === AUTHOR_NAME
  # ========================================================================= #
  AUTHOR_NAME = RBT.author_name? # your name

  # ========================================================================= #
  # === DEFAULT_FONT_SIZE
  # ========================================================================= #
  DEFAULT_FONT_SIZE = '16'

  # ========================================================================= #
  # === USE_THIS_MONOSPACED_FONT
  # ========================================================================= #
  USE_THIS_MONOSPACED_FONT = :hack_16

  # ========================================================================= #
  # === ARRAY_WITH_PROGRAM_DIRS
  # ========================================================================= #
  ARRAY_WITH_PROGRAM_DIRS = [
    '','RB', 'PHP', 'PYTHON','AUD',
    'VID',
    'EDITORS',
    'GCC',
    'GNOME',
    'IMG',
    'GAMES',
    'KDE'
  ]

  # ========================================================================= #
  # === ARRAY_AVAILABLE_PROGRAMS
  # ========================================================================= #
  ARRAY_AVAILABLE_PROGRAMS = ::RBT.available_programs?

  # ========================================================================= #
  # === initialize
  #
  # The second argument is the text which can be shown on the
  # compile-button. The default text is 'Compile'. The reason why
  # this option exists is because we may want to have this widget
  # 'Install' rather than 'Compile' sometimes.
  # ========================================================================= #
  def initialize(
      commandline_arguments       = ARGV,
      run_already                 = true
    )
    super(:vertical) if use_gtk3?
    determine_the_GUI_to_be_used(commandline_arguments) # This must come first, even before reset().
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    on_delete_event_quit_the_application
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super() if respond_to?(:super)
    reset_the_shared_module # This can come early.
    reset_the_base_module
    reset_the_internal_variables
    infer_the_namespace
    # ======================================================================= #
    # === @configuration
    # ======================================================================= #
    @configuration = [true, __dir__, namespace?]
    # ======================================================================= #
    # === Set the title, width, height and the font in use.
    # ======================================================================= #
    title_width_height_font(TITLE, WIDTH, HEIGHT, USE_THIS_FONT)
    if use_gtk3?
      handle_CSS_rules
    end
    infer_the_size_automatically
  end

  # ========================================================================= #
  # === favicon?
  # ========================================================================= #
  def favicon?
    :tabble
  end

  # ========================================================================= #
  # === smaller_font?
  # ========================================================================= #
  def smaller_font?
    SMALLER_FONT
  end

  # ========================================================================= #
  # === handle_CSS_rules                                            (CSS tag)
  # ========================================================================= #
  def handle_CSS_rules
    use_gtk_paradise_project_css_file
    append_project_css_file
    more_CSS_then_apply_it(USE_THESE_CSS_RULES)
  end

  # ========================================================================= #
  # === reset_the_shared_module
  #
  # This method can be used for ruby-gtk3 and ruby-libui, among other
  # toolkits.
  # ========================================================================= #
  def reset_the_shared_module
  end

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    0
  end

  # ========================================================================= #
  # === border_size?
  # ========================================================================= #
  def border_size?
    2
  end

  # ========================================================================= #
  # === do_save_the_buffer                                        (save tag)
  # ========================================================================= # 
  def do_save_the_buffer(
      what = main_buffer?.text.to_s
    )
    into = RBT.log_directory?+
           main_entry?.text.to_s+'.yml'
    e "Saving into `#{into}`."
    RBT.write_what_into(what, into)
    return into
  end

  # ========================================================================= #
  # === do_assign_data_from_this_program
  # ========================================================================= #
  def do_assign_data_from_this_program(
      this_program = @entry_name_of_the_program.text?
    )
    # ======================================================================= #
    # We have to check whether the program is included in RBT or
    # whether it is not.
    # ======================================================================= #
    if ::RBT.is_this_program_included?(this_program)
      mass_append(return_hash_from_sanitize_cookbook(this_program))
    else
      e "The program `#{this_program}` is not included."
    end
  end; alias update_the_main_buffer do_assign_data_from_this_program # === update_the_main_buffer

  # ========================================================================= #
  # === main_entry?
  # ========================================================================= #
  def main_entry?
    @entry_name_of_the_program
  end

  # ========================================================================= #
  # === set_handle_this_program
  # ========================================================================= #
  def set_handle_this_program(i)
    @entry_name_of_the_program.set_text(i)
    update_the_main_buffer
  end

  # ========================================================================= #
  # === return_hash_from_sanitize_cookbook
  # ========================================================================= #
  def return_hash_from_sanitize_cookbook(
      for_this_program = @entry_name_of_the_program.text?
    )
    _ = RBT::Cookbooks::SanitizeCookbook.new(for_this_program)
    binaries = _.binaries?
    hash = {
      short_name: _.short_name?,
      binaries: binaries,
      program_name_and_program_version: _.program_name_and_program_version?,
      short_description: _.short_description?,
      description: _.description?,
      extra_information: _.extra_information?,
      licence: _.licence?,
      url1: _.url1?,
      url2: _.url2?,
      homepage: _.homepage?,
      git_url: _.git_url?,
      prefix: _.prefix?,
      use_autogen: _.use_autogen?,
      keep_extracted: _.keep_extracted?,
      use_build_directory: _.use_build_directory?,
      tags: _.tags?,
      required_deps_on: _.required_deps_on?,
      last_update: _.last_update?
    }
    return hash
  end

  require 'rbt/toplevel_methods/determine_appdir_prefix_from_this_input_by_guessing_the_program_version.rb'
  # ========================================================================= #
  # === create_the_buttons
  # ========================================================================= #
  def create_the_buttons
    # ======================================================================= #
    # === The traditional button aka prefix: t
    # ======================================================================= #
    @button_traditional = check_button('_Traditional')
    @button_traditional.hint = 'This button can be used to toggle between '\
      'the /usr/ prefix and the corresponding app-dir prefix.'
    @button_traditional.on_toggled { |widget|
      this_program = compile_this_program?
      hash = return_hash_from_sanitize_cookbook(this_program)
      if widget.active?
        hash[:prefix] = '/usr/'
      else
        hash[:prefix] =
          RBT.determine_appdir_prefix_from_this_input_by_guessing_the_program_version(this_program)
      end
      mass_append(hash)
    }
    # ======================================================================= #
    # === The save-button is created next
    # ======================================================================= #
    @button_save = button('_Save')
    @button_save.hint = 'Click this button in order to save the content '\
      'of the text view component, below this widget. This will save '\
      'within the directory <b>'+RBT.log_directory?+'</b>.'
    @button_save.bblack1
    @button_save.on_clicked {
      location = do_save_the_buffer
      pop_up_this_message_over_that_widget(location, @button_save)
    }
    # ======================================================================= #
    # === The clear-button is created next
    # ======================================================================= #
    @button_clear = button('_Clear')
    @button_clear.hint = 'Click this button in '\
      'order to clear the content of the text view component, '\
      'below this widget.'
    @button_clear.bblack1
    @button_clear.on_clicked {
      @entry_name_of_the_program.clear
      main_text_view?.clear
    }
    # ======================================================================= #
    # === The download button comes next
    #
    # This button can be used to quickly download the program at hand.
    # ======================================================================= #
    @button_download = button('_Download' )
    @button_download.hint = 'Click this button in '\
      'order to download the remote URL associated with the '\
      'given program at hand. This only works for programs '\
      'that are registered within the RBT project, though.'
    @button_download.bblack1
    @button_download.on_clicked {
      download_program(RBT.return_url1_of(handle_which_program?))
    }
  end

  # ========================================================================= #
  # === program_name?
  # ========================================================================= #
  def program_name?
    @entry_name_of_the_program.text?
  end; alias compile_this_program? program_name? # === compile_this_program?
       alias handle_which_program? program_name? # === handle_which_program?

  # ========================================================================= #
  # === download_program
  #
  # This will download a program by using our Download module.
  # ========================================================================= #
  def download_program(
      url = RBT.return_url1_of(handle_which_program?)
    )
    e "Downloading #{url} next, into the directory at #{sfancy(Dir.pwd)}:"
    Cliner.cliner {
      Wget.download(url)
    }
  end

  # ========================================================================= #
  # === append
  #
  # This will, by default, append via a newline. See the second argument
  # to this method.
  # ========================================================================= #
  def append(
      i,
      append_this_string = "\n"
    )
    @text_buffer.append(
      "#{i}#{append_this_string}"
    )
  end

  # ========================================================================= #
  # === do_process_the_commandline_arguments
  #
  # For now the first argument given is assumed to be a program.
  # ========================================================================= #
  def do_process_the_commandline_arguments
    _ = commandline_arguments?
    if _ and !_.empty?
      set_handle_this_program(_.first)
    end
  end

  # ========================================================================= #
  # === main_buffer?
  # ========================================================================= #
  def main_buffer?
    @text_buffer
  end

  # ========================================================================= #
  # === colourize_the_buffer
  #
  # This method will colourize the buffer at hand.
  # ========================================================================= #
  def colourize_the_buffer
    _ = @text_buffer.text?
    @text_buffer.clear
    iter = @text_buffer.start_iter
    _.split(N).each { |line|
      case line
      when /^ last_update:/i
        @text_buffer.insert(iter, line+N,tags: ['bold','tomato'])
      # ===================================================================== #
      # === name:
      # ===================================================================== #
      when /name:$/,
           /^#{program_name?}:$/
        @text_buffer.insert(iter, line+N,tags: ['bold','seagreen'])
      # ===================================================================== #
      # === program_name
      # ===================================================================== #
      when /program_name/
        @text_buffer.insert(iter, line+N,tags: ['bold','darkblue'])
      # ===================================================================== #
      # === description
      # ===================================================================== #
      when /description/
        @text_buffer.insert(iter, line+N,tags: ['bold','darkgreen'])
      # ===================================================================== #
      # === required_deps
      # ===================================================================== #
      when /required_deps/
        @text_buffer.insert(iter, line+N,tags: ['bold','slateblue'])
      # ===================================================================== #
      # === true
      # ===================================================================== #
      when /true/, / t$/
        line.gsub!(/ true/,'')
        line.gsub!(/ t$/,'') if line =~ / t$/
        @text_buffer.insert(iter, line)
        @text_buffer.insert(iter, ' yes'+N, tags: ['bold','red'])
      # ===================================================================== #
      # === false
      # ===================================================================== #
      when /false/, / f$/
        line.gsub!(/ false/,'')
        line.gsub!(/ f$/,'') if line =~ / f$/
        @text_buffer.insert(iter, line)
        @text_buffer.insert(iter, ' no'+N, tags: ['bold','red'])
      else
        @text_buffer.insert(iter, line+N)
      end
    }
  end

  # ========================================================================= #
  # === main_text_view?
  # ========================================================================= #
  def main_text_view?
    @text_view
  end

  # ========================================================================= #
  # === create_the_skeleton                        (create tag, skeleton tag)
  # ========================================================================= #
  def create_the_skeleton
    create_the_buttons
    create_the_text_buffer
  end

  # ========================================================================= #
  # === create_a_new_about_dialog
  #
  # This creates our about dialog.
  # ========================================================================= #
  def create_a_new_about_dialog
    @about_dialog = gtk_about_dialog
    @about_dialog.artists = [ AUTHOR_NAME ]
    @about_dialog.authors = [ AUTHOR_NAME ]
    @about_dialog.comments = 'This Script creates recipe information '\
      'which will be stored in sources.yaml'
    @about_dialog.copyright = 'None'
    @about_dialog.documenters = ['']
    @about_dialog.license = 'This program is licenced under the same license '\
      'as Ruby. There is one additional license part - which simply '\
      'encourages you to send bug reports to '\
      'me so that I can improve the program. ;-)'
    @about_dialog.name = 'Ruby::Gtk::ReciperMaker'
    # @about_dialog.translator_credits = "Here you can insert the credits of the translator.\n"
    @about_dialog.version = VERSION_RECIPE_MAKER
    @about_dialog.website = 'https://rubygems.org/gems/rbt'
    @about_dialog.website_label = 'RBT on rubygems.org'
    _ = ENV['IMG'].to_s+'/STD/UHR.png'
    if File.exist? _
      @about_dialog.logo = GdkPixbuf::Pixbuf.new(file: _)
    end
    @about_dialog.popup
  end
  
  # ========================================================================= #
  # === do_open_a_local_recipe                                     (open tag)
  # ========================================================================= #
  def do_open_a_local_recipe
    create_a_new_file_chooser_dialog(self) {{
      start_dir:              ::RBT.project_base_directory?+'yaml/cookbooks/',
      additional_directories: ::RBT.project_base_directory?+'yaml/cookbooks/'
    }}
    _ = ::Gtk.main_file?
    if _ and File.exist?(_)
      @entry_name_of_the_program.set_text(File.basename(_).delete_suffix('.yml'))
      do_assign_data_from_this_program
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    run_super
  end

  # ========================================================================= #
  # === connect_the_skeleton                                    (connect tag)
  # ========================================================================= #
  def connect_the_skeleton
    abort_on_exception

    outer_vbox = create_vbox # A new vbox.

    _ = default_top_bar
    _.title = 'Recipe maker - create a new .yml file, a recipe'
    _.css_class('pad0px')
    # ======================================================================= #
    # === Create the open-file-button
    # ======================================================================= #
    @button_open_file = create_button('Open File')
    @button_open_file.bblack1
    @button_open_file.hint = 'Click on this button in order to '\
                             'select a local .cfdg file to open.'
    @button_open_file.on_clicked {
      do_open_a_local_recipe
    }
    _.pack_start(@button_open_file)
    # ======================================================================= #
    # Add the about-dialog next.
    # ======================================================================= #
    button_about_dialog = button('About')
    button_about_dialog.bblack1
    button_about_dialog.on_clicked {
      create_a_new_about_dialog
    }
    _.pack_end(button_about_dialog)
    outer_vbox.minimal(_)
    hbox_for_the_name_of_the_program = create_hbox
    hbox_for_the_name_of_the_program.minimal(
      text('Input the name of the program:', 2)
    )
    @entry_name_of_the_program = entry('htop')
    @entry_name_of_the_program.default_entry
    @entry_name_of_the_program.pad8px
    @entry_name_of_the_program.on_click_select_everything
    @entry_name_of_the_program.width_height(250, 50)
    @entry_name_of_the_program.center
    # @entry_name_of_the_program.set_completion(
    #   ARRAY_AVAILABLE_PROGRAMS
    # ) if @entry_name_of_the_program.respond_to? :set_completion
    @entry_name_of_the_program.do_focus
    @entry_name_of_the_program.on_enter {
      do_assign_data_from_this_program
    }
    hbox_for_the_name_of_the_program.minimal(@entry_name_of_the_program, 2)
    # === button_assign_data_to_the_buffer
    button_assign_data_to_the_buffer = button('_Assign')
    button_assign_data_to_the_buffer.hint = 'Click this button in '\
      'order to assign to the text view below this widget. Note that '\
      'this will only work if that program is registered in the '\
      'RBT project.'
    button_assign_data_to_the_buffer.bblack1
    button_assign_data_to_the_buffer.on_clicked {
      do_assign_data_from_this_program(@entry_name_of_the_program.text?)
    }
    hbox_for_the_name_of_the_program.minimal(button_assign_data_to_the_buffer, 2)
    hbox_for_the_name_of_the_program.minimal(@button_clear, 2)
    hbox_for_the_name_of_the_program.minimal(@button_traditional, 2)
    hbox_for_the_name_of_the_program.minimal(@button_download, 2)
    hbox_for_the_name_of_the_program.minimal(@button_save, 2)

    outer_vbox.minimal(hbox_for_the_name_of_the_program, 4)
    outer_vbox.minimal(@outermost_vbox, 1)

    window = create_window_or_runner
    window << outer_vbox

    properly_prepare_this_window(window,
      {
        title:       title?,
        font:        font?,
        width:       width?,
        height:      height?,
        padding:     padding?,
        border_size: border_size?
      }
    )
    window.show_all
    window.top_left
    do_all_startup_related_actions
    run_main
  end

  # ========================================================================= #
  # === create_the_text_buffer
  #
  # This method will create the textedit stuff.
  # ========================================================================= #
  def create_the_text_buffer
    @outermost_vbox = create_vbox
    @text_buffer = create_text_buffer
    if use_gtk3?
      require 'gtk_paradise/toplevel_methods/tokenizer_and_text_buffer_tags.rb'
      @text_buffer = ::Gtk::TextBufferTags.create_default_tags(@text_buffer)
    end
    @text_view = create_text_view(@text_buffer)
    # @text_view.it_can_not_be_edited
    @text_view.use_this_font = USE_THIS_MONOSPACED_FONT
    hbox = create_hbox
    _ = return_widget_with_line_numbers {{ n_lines: 40 }}
    _.use_this_font = USE_THIS_MONOSPACED_FONT
    hbox.minimal(_)
    hbox.maximal(@text_view)

    another_scrolled_window = create_scrolled_window(hbox) { :always }
    another_scrolled_window.set_policy(:never, :always)
    another_scrolled_window.bblack3
    another_scrolled_window.width_height(720, 520)
    another_scrolled_window.pad5px
    @outermost_vbox.minimal(another_scrolled_window, 1)
  end

  # ========================================================================= #
  # === mass_append
  #
  # The first argument to this method must be a Hash that contains all
  # relevant data.
  #
  # Whenever this method is used, the old text-buffer is cleared.
  # ========================================================================= #
  def mass_append(hash)
    @text_buffer.clear # Clear the old content first.
    append(hash[:short_name].to_s+':') # This entry is mandatory.
    append(' # blfs:')
    append(' # -')
    unless hash[:binaries].empty?
      append(' binaries:')
      append(' - ')
    end
    append(
      ' # program_name_and_program_version: '+hash[:program_name_and_program_version]
    )
    append(' configure_options: |')
    append('  enable-shared')
    if hash.has_key? :short_description
      append(' short_description: |')
      append('  '+hash[:short_description].to_s)
    end
    append(' description: |')
    append(
      '  '+RBT.word_wrap(hash[:description], 60)
    )
    append(' extra_information: |')
    append('  '+hash[:extra_information].to_s)
    append(' licence: '+hash[:licence].to_s)
    append(' url1: '+hash[:url1].to_s)
    append(' url1: '+hash[:url2].to_s)
    append(' homepage: '+hash[:homepage].to_s)
    append(' git_url: '+hash[:git_url].to_s) if hash[:git_url]
    append(' prefix: '+hash[:prefix].to_s)
    append(' use_autogen: '+hash[:use_autogen].to_s)
    append(' keep_extracted: '+hash[:keep_extracted].to_s)
    append(' use_build_directory: '+hash[:use_build_directory].to_s)
    if hash[:tags] and hash[:tags].is_a? Array
      append(' tags:')
      hash[:tags].each {|inner_entry|
        append('  - '+inner_entry.to_s)
      }
    end
    append(' required_deps_on:')
    hash[:required_deps_on].each {|this_dependency|
      append('  - '+this_dependency)
    }
    append(' last_update: '+hash[:last_update].to_s)
    colourize_the_buffer
  end

  # ========================================================================= #
  # === do_all_startup_related_actions
  # ========================================================================= #
  def do_all_startup_related_actions
    do_process_the_commandline_arguments
    cd(RBT.log_directory) # Must navigate to a "safe directory".
    Thread.new {
      sleep 0.1
      do_assign_data_from_this_program
    }
  end

  # ========================================================================= #
  # === RBT::GUI::Gtk::RecipeMaker.run
  # ========================================================================= #
  def self.run(
      i = ARGV
    )
    require 'gtk_paradise/run'
    _ = ::RBT::GUI::Gtk::RecipeMaker.new(i)
    r = ::Gtk.run
    _.set_parent_widget(r) # Must come before we enable the key-combinations.
    r << _
    r.add_shortcut(1,  'focus_on_entry', :alt)
    r.add_shortcut(:a, 'focus_on_entry', :alt)
    r.automatic_size_then_automatic_title
    r.enable_quick_exit
    r.set_background :white
    r.is_resizable # <- This line was added due to a possible prior .maximize action.
    r.top_left_then_run
  end; self.instance_eval { alias start_gui_application run } # === RBT::GUI::Gtk::RecipeMaker.start_gui_application

  # ========================================================================= #
  # === RBT::GUI::UniversalWidgets::RecipeMaker[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::GUI::UniversalWidgets::RecipeMaker.new(ARGV)
end # gui_rbt