#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CreateDebianControlFile
#
# This class can generate a control-file for use in debian.
#
# The control-file contains various values which dpkg, dselect, apt-get,
# apt-cache, aptitude, and other package management tools will use to
# manage the package, on a debian system.
#
# It is defined by the Debian Policy Manual, 5 "Control files and
# their fields".
#
# Note: this class is presently unfinished.
# =========================================================================== #
# require 'rbt/linux/debian/create_control_file.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Linux

module Debian

class CreateControlFile < Base # === RBT::Linux::Debian::CreateControlFile

  # ========================================================================= #
  # === initialize
  #
  # The first argument is the name of the program.
  # ========================================================================= #
  def initialize(
      full_name_of_the_program = return_pwd, # By default we will obtain information from the current working directory.
      run_already         = true
    )
    reset
    set_full_name_of_the_program(
      full_name_of_the_program
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @_
    # ======================================================================= #
    @_ = ''.dup # <- This is the string that will be stored.
    # ======================================================================= #
    # === @filename
    # ======================================================================= #
    @filename = 'control'
  end

  # ========================================================================= #
  # === set_full_name_of_the_program
  # ========================================================================= #
  def set_full_name_of_the_program(i)
    if i.is_a? Array
      i = i.first
    end
    case i
    when nil, :default
      i = return_pwd
    end
    @full_name_of_the_program = File.basename(i)
  end

  # ========================================================================= #
  # === create_the_control_file
  # ========================================================================= #
  def create_the_control_file
    opne 'Now storing in the file `'+sfile(@filename)+'`.'
    write_what_into(@_, @filename)
  end

  # ========================================================================= #
  # === append_the_package_content_onto_the_main_string
  # ========================================================================= #
  def append_the_package_content_onto_the_main_string
    _ = ProgramInformation[@full_name_of_the_program]
    name_of_the_program    = _.name_of_the_program?
    version_of_the_program = _.version_of_the_program?
    heredoc = <<-EOF
Package: #{name_of_the_program}
Version: #{version_of_the_program}
Section: base
Priority: optional
Architecture: i386
Depends: libsomethingorrather (>= 1.2.13), anotherDependency (>= 1.2.6)
Maintainer: Your Name <#{RBT.email?}>
Description: Hello World
 Foo
 Bar
EOF
    append(heredoc)
  end

  # ========================================================================= #
  # === append
  # ========================================================================= #
  def append(i)
    @_ << i
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    append_the_package_content_onto_the_main_string
    create_the_control_file
  end

  # ========================================================================= #
  # === RBT::Linux::Debian::CreateControlFile[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Linux::Debian::CreateControlFile.new(ARGV)
end # createdebiancontrolfile