#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CreateRpmSpecFile
#
# This class can be used to create a rpm-spec file.
#
# Usage example:
#
#   RBT::CreateRpmSpecFile.new(ARGV)
#
# =========================================================================== #
# require 'rbt/package_managers/create_rpm_spec_file.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CreateRpmSpecFile < Base # === RBT::CreateRpmSpecFile

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @name_of_the_program
    # ======================================================================= #
    @name_of_the_program = nil
    # ======================================================================= #
    # === @description
    # ======================================================================= #
    @description = nil
  end

  # ========================================================================= #
  # === determine_the_name_of_the_file
  # ========================================================================= #
  def determine_the_name_of_the_file
    @name_of_the_program = first?.to_s.downcase
    @name_of_the_file = @name_of_the_program+'.spec'
  end

  # ========================================================================= #
  # === description?
  # ========================================================================= #
  def description?
    @dataset.description?
  end

  # ========================================================================= #
  # === return_program_version
  # ========================================================================= #
  def return_program_version
    @dataset.program_version?.to_s
  end

  # ========================================================================= #
  # === return_licence
  # ========================================================================= #
  def return_licence
    @dataset.licence?.to_s
  end

  # ========================================================================= #
  # === return_short_desc
  # ========================================================================= #
  def return_short_desc
    @dataset.short_desc?.to_s
  end

  # ========================================================================= #
  # === load_the_cookbook_dataset
  # ========================================================================= #
  def load_the_cookbook_dataset
    @dataset = RBT.cookbook(@name_of_the_program)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    determine_the_name_of_the_file
    load_the_cookbook_dataset
    _ = '
Name:       '+File.basename(@name_of_the_program)+'
Version:    '+return_program_version+'
Release:    1
Summary:    '+return_short_desc+'
License:    '+return_licence+'

%description
'+description?.to_s+'

%prep
# we have no source, so nothing here

%build
cat > hello-world.sh <<EOF
#!/usr/bin/bash
echo Hello world
EOF

%install
mkdir -p %{buildroot}/usr/bin/
install -m 755 hello-world.sh %{buildroot}/usr/bin/hello-world.sh

%files
/usr/bin/hello-world.sh

%changelog
# lets skip this for now
'
    into = log_dir?+@name_of_the_file
    opne "Storing into #{sfile(into)} next."
    write_what_into(_, into)
  end

  # ========================================================================= #
  # === RBT::CreateRpmSpecFile[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::CreateRpmSpecFile.new(ARGV)
end # createrpmspecfile viennarna