#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::RegisterEverything
# =========================================================================== #
# require 'rbt/registered/register_everything.rb'
# RBT.register_everything
# =========================================================================== #
require 'rbt/registered/registered.rb'

module RBT

module Cookbooks

class RegisterEverything < RBT::Cookbooks::Registered

  require 'rbt/registered/registered_libraries.rb'
  require 'rbt/registered/registered_pkgconfig_files.rb'
  require 'rbt/registered/registered_headers.rb'
  require 'rbt/registered/registered_binaries.rb'
  require 'rbt/registered/registered_tags.rb'
  require 'rbt/registered/registered_m4_files.rb'
  require 'rbt/registered/registered_gir_files.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize
    reset
    run
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    RBT::Cookbooks::RegisteredLibraries.new
    RBT::Cookbooks::RegisteredPkgconfigFiles.new
    RBT::Cookbooks::RegisteredHeaders.new
    RBT::Cookbooks::RegisteredBinaries.new
    RBT.create_registered_tags
    RBT::Cookbooks::RegisteredM4Files.new
    RBT::Cookbooks::RegisteredGirFiles.new
  end

end; end

# =========================================================================== #
# === RBT.register_everything
# =========================================================================== #
def self.register_everything
  RBT::Cookbooks::RegisterEverything.new
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::RegisterEverything.new
end