#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/sinatra/action_view.rb'
# =========================================================================== #
module RBT

class Sinatra < ::Sinatra::Base

  begin
    require 'cyberweb/requires/require_the_html_template.rb'
  rescue LoadError; end

  # ========================================================================= #
  # === /view
  #
  # http://localhost:5580/view
  # ========================================================================= #
  get('/view') {
    route_to_this_action = '/view/'
    body = div(css_style: 'padding: 0.2em') {
      h1('<b>Display this program:</b>')+
      p(
        'This will display information about a (registered) program,
        such as "<b>htop</b>", without the quotes.<br>
        (In order for this to work, the program must have been
        <b>registered</b> in the RBT project.)',
        css_style: 'padding:0.55em; margin: 1px;'
      )+
      '<form id="view" style="margin-left:2em; margin-top:2px" action="'+route_to_this_action+'">
       <input type="text" name="user_input" style="border:3px solid slateblue; padding: 4px"><br>
       <input type="submit" name="user_input_submit" value="Display" style="2px dotted royalblue; '\
       'color: white; background-color: olive; font-size: 1.50em; margin:2px; padding: 4px">
       </form>'+
      return_to_root_link
    }
    return Cyberweb::HtmlTemplate[
      title: 'View the following package',
      body:  body
    ].to_s
  }

  # ========================================================================= #
  # === /view/*                                                    (view tag)
  #
  # This method will allow us to show content about a particular
  # program, such as "php" or "htop" or "ruby".
  #
  # Usage examples:
  #
  #   http://localhost:5580/view/php
  #   http://localhost:5580/view/ruby
  #   http://localhost:5580/view/?user_input=ruby&user_input_submit=Display
  #
  # ========================================================================= #
  get('/view/*'){
    use_these_params = params[:splat]
    if params.has_key?('user_input')
      use_these_params = params.fetch('user_input')
    end
    result = ''.dup
    if RBT.does_include? use_these_params
      _ = RBT::Cookbooks::SanitizeCookbook.new(use_these_params) { :fast }
      program_path = _.program_path?
      description  = _.raw_description?
      program_name = _.program_name?
      binaries     = _.binaries?
      properly_formatted_description = description.to_s.tr("\n",'<br>')
      properly_formatted_extra_information = _.extra_information?.to_s
      result << return_html_head_and_title_and_body+p(
        '<b>Program name</b>: '+program_name.to_s+"<br>\n"+
        '<b>Program version</b>: '+_.program_version?.to_s+"<br>\n"+
        '<b>Program path</b>: '+program_path.to_s+"<br>\n"+
        '<b>Description</b>:<br>'+
        '<p style="margin-bottom: 5px; margin-left: 1.0em; margin-top: 5px;
         width:80%; padding: 0.5em">'+
        properly_formatted_description+
        '</p>'
      )
      # ===================================================================== #
      # === Add the extra information next
      # ===================================================================== #
      if properly_formatted_extra_information and
        !properly_formatted_extra_information.empty?
        result << '<b>Extra information:</b><br>'
        result << p(properly_formatted_extra_information,
          css_style:
          'margin-bottom: 5px;
           margin-left: 2.0em;
           margin-top: 5px;
           width:80%;
           padding: 0.5em'
        )
      end
      # ===================================================================== #
      # === Add which binaries will be installed by this program:
      # ===================================================================== #
      if binaries and !binaries.empty?
        result << 'The following <b>binaries</b> will be installed:<br><br>'
        binaries.each {|this_binary|
          result << "<span style=\"padding-left: 2em\"> - <b>#{this_binary}</b></span><br>\n"
        }
        result << '<br>'
      end
      result << '<b class="darkgreen">Last update:</b> <b>'+
                _.last_update?.to_s+'</b><br>'+"\n"
      result << '<br>'
    else
      if use_these_params.is_a? Array
        use_these_params = use_these_params.join(' ').strip
      end
      if use_these_params.empty?
        result << p(
          'Please supply a program name.'
        )
      else
        result << p(
          'The program <b>'+use_these_params+'</b> is <b>not</b> included.'
        )
      end
    end
    if use_these_params.is_a? Array
      use_these_params = use_these_params.first
    end
    Cyberweb::HtmlTemplate[
      css_class: 'darkgreen',
      title: 'Displaying information about '+use_these_params.to_s,
      body: result
    ].to_s
  }

end; end