#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CompileViaSQL
#
# This class has probably been superseded by class 
# RBT::CompileViaSQLDatabase in August 2022. However
# had, for the time being, we will keep this class
# and decide at a later time what to do with it
# (or with the other class).
#
# Usage example:
#
#   RBT::CompileViaSQL.new(ARGV)
#
# =========================================================================== #
# require 'rbt/sql/compile_via_sql.rb'
# RBT::CompileViaSQL.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CompileViaSQL < Base # === RBT::CompileViaSQL

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    try_to_require_the_extracter_gem
    # ======================================================================= #
    # === @use_this_database
    # ======================================================================= #
    @use_this_database = '/Depot/jjjj/registered_programs.db'
    if File.exist? 'registered_programs.db'
      @use_this_database = File.absolute_path('registered_programs.db')
    end
  end

  # ========================================================================= #
  # === use_this_database?
  # ========================================================================= #
  def use_this_database?
    @use_this_database
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = first_argument?
    query = "sqlite3 #{use_this_database?} \"SELECT PROGRAM_PATH FROM REGISTERED_PROGRAMS WHERE PROGRAM_NAME='#{_}';\""
    e steelblue(query)
    program_path = `#{query}`.strip
    query2 = "sqlite3 #{use_this_database?} \"SELECT EXTRACT_TO FROM REGISTERED_PROGRAMS WHERE PROGRAM_NAME='#{_}';\""
    extract_to = `#{query2}`.strip
    e 'Extracting '+sfancy(program_path)+
      ' towards '+sdir(extract_to)+' next:'
    Extracter.what_to(
      program_path,
      File.dirname(extract_to)
    )
    cd(extract_to)
    esystem('./configure --prefix=/usr')
    esystem('make')
    esystem('make install')
  end

  # ========================================================================= #
  # === RBT::CompileViaSQL[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::CompileViaSQL.new(ARGV)
end # compileviasql php