#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/all_libraries.rb'
# =========================================================================== #
module RBT

  require 'rbt/project/project.rb'
  require 'rbt/requires/require_yaml.rb'
  require 'rbt/constants/constants.rb'

  # ========================================================================= #
  # === FILE_REGISTERED_LIBRARIES
  #
  # This may be a file path such as:
  #
  #   /Programs/Ruby/2.4.1/lib/ruby/site_ruby/2.4.0/rbt/yaml/registered_libraries.yml
  #
  # cat $RBT/yaml/registered_libraries.yml
  # ========================================================================= #
  FILE_REGISTERED_LIBRARIES =
    "#{RBT.yaml_directory_for_the_registered_yaml_files?}libraries.yml"

  # ========================================================================= #
  # === YAML_FILE_REGISTERED_LIBRARIES
  #
  # This is equivalent to:
  #   data = YAML.load_file(RBT.yaml_directory?+'registered_libraries.yml')
  # ========================================================================= #
  if File.exist? FILE_REGISTERED_LIBRARIES
    YAML_FILE_REGISTERED_LIBRARIES = YAML.load_file(FILE_REGISTERED_LIBRARIES)
  else
    YAML_FILE_REGISTERED_LIBRARIES = {} # Otherwise, initialize as empty hash.
  end; ALL_LIBRARIES = YAML_FILE_REGISTERED_LIBRARIES # === ALL_LIBRARIES

  # ========================================================================= #
  # === RBT.file_registered_libraries
  # ========================================================================= #
  def self.file_registered_libraries
    "#{RBT.yaml_directory_for_the_registered_yaml_files?}libraries.yml"
  end; self.instance_eval { alias location_registered_libraries file_registered_libraries } # === RBT.location_registered_libraries

  # ========================================================================= #
  # === RBT.all_libraries?
  #
  # Get back all registered libraries.
  #
  # This is guaranteed to return a Hash.
  # ========================================================================= #
  def self.all_libraries?
    ALL_LIBRARIES
  end; self.instance_eval { alias all_libraries all_libraries? } # === RBT.all_libraries

  # ========================================================================= #
  # === RBT.this_library_to_that_program
  #
  # Usage example:
  #
  #   RBT.this_library_to_that_program('libc.so') # => "musl"
  #
  # ========================================================================= #
  def self.this_library_to_that_program(i)
    ALL_LIBRARIES[i]
  end

  # ========================================================================= #
  # === all_libraries?
  # ========================================================================= #
  def all_libraries?
    RBT.all_libraries?
  end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  require 'colours'
  pp RBT.all_libraries?
  e
  e "#{Colours.rev}This was read from the file:"
  e
  e Colours.sfancy("  #{RBT.file_registered_libraries}")
  e
end # alllibraries?