#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/chroot_and_chown_related_code.rb'
# RBT.chroot_compile(ARGV)
# =========================================================================== #
module RBT

  require 'rbt/constants/constants.rb'
  require 'rbt/toplevel_methods/files_and_directories_related_methods.rb'

  # ========================================================================= #
  # === RBT.chroot_compile
  #
  # This method can be used to chroot-compile a given program, that is,
  # to compile it INTO the chroot. The default prefix for that chroot
  # environment will be /usr/. This is currently a hardcoded path.
  #
  # This is mostly a "hackish" method in that I needed it to quickly
  # compile programs into my main chroot, for when I build it up.
  # ========================================================================= #
  def self.chroot_compile(
      this_program       = ARGV,
      use_this_as_prefix = '/Depot/Chroot/usr/' # The default prefix target.
    )
    require 'rbt/requires/require_actions.rb'
    if this_program.is_a? Array
      this_program.flatten!
      this_program.compact!
      this_program.map! {|entry|
        if entry =~ /^\d+$/ # If it is a number only, such as 1, 2, 3 etc.
          file_installation_procedure = RBT.file_installation_procedere
          if File.exist? file_installation_procedure
            dataset = YAML.load_file(file_installation_procedere)['default']
            entry = dataset[entry.to_i - 1]
          else
            no_file_exists_at(file_installation_procedure)
          end
        end
        entry
      }
    end
    # ======================================================================= #
    # Finally delegate towards class RBT::Compile.
    # ======================================================================= #
    action(:Installer, program) {{
      use_this_as_prefix: use_this_as_prefix
    }}
  end; self.instance_eval { alias uchroot_compile chroot_compile } # === RBT.chroot_compile

  # ========================================================================= #
  # === RBT.fix_chown_root_entries_in_this_file
  #
  # This method can be used to replace all "chown root:root" entries
  # with "chown 0:0" entries. It may be necessary for when we have
  # setup a chroot.
  # ========================================================================= #
  def self.fix_chown_root_entries_in_this_file(
      this_file = 'Makefile'
    )
    what = ''.dup
    if File.exist? this_file
      File.readlines(this_file).each {|this_entry|
        if this_entry.include? 'chown root:root'
          this_entry.gsub!(/root:root/, '0:0')
        end
        what << this_entry
      }
      e "#{rev}Now modifying the file `#{sfile(this_file)}#{rev}`."
      write_what_into(what, this_file)
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.chroot_compile(ARGV)
end # chrootcompile gettext