#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/cliner.rb'
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/e_and_esystem.rb'

  # ========================================================================= #
  # === RBT.unicode_cliner
  #
  # This is like RBT.cliner(), but it will make use of unicode horizontal
  # building block.
  # ========================================================================= #
  def self.unicode_cliner(
      use_this_token  = '=',
      use_this_colour = nil,
      &block
    )
    yielded = :unicode
    if block_given?
      yielded = yield
    end
    RBT.cliner(use_this_token, use_this_colour) { yielded }
  end

  # ========================================================================= #
  # === RBT.cliner
  #
  # This method accepts block-parameters such as :unicode, which
  # will be "converted" into a corresponding horizontal line via
  # a certain unicode block-symbol.
  #
  # Usage example:
  #
  #   RBT.cliner('_', :steelblue)
  #
  # ========================================================================= #
  def self.cliner(
      use_this_token  = '=',
      use_this_colour = nil,
      &block
    )
    case use_this_token
    # === :do_not_use_colours
    when :do_not_use_colours
      use_this_token = '='
      use_this_colour = nil
    # === :default
    when :default
      use_this_token = '='
    end
    case use_this_colour
    when :default
      use_this_colour = nil
    end
    # ======================================================================= #
    # === Handle blocks given to this method next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      when :unicode_middle_horizontal_bar
        begin
          # ================================================================= #
          # Fetch the unicode block here.
          # ================================================================= #
          require 'roebe/requires/require_unicode.rb'
          use_this_token = ::Roebe.horizontal_double_bar
        rescue LoadError
        end
      when :unicode
        begin
          # ================================================================= #
          # Fetch the unicode block here.
          # ================================================================= #
          require 'roebe/requires/require_unicode.rb'
          use_this_token = ::Roebe.return_horizontal_unicode_block
        rescue LoadError
        end
      end
    end
    if use_this_token.is_a? Hash
      if use_this_token.has_key? :use_colour
        use_this_colour = use_this_token[:use_colour]
        use_this_token = '=' # Restore the default in this case.
      end
    end
    case use_this_token
    when nil, :default
      use_this_token = '=' # <- Restore to the default in this case.
    end
    if use_this_colour
      # Support e. g. "konsole_dodgerblue" next.
      if use_this_colour.is_a?(Symbol) or
        ( use_this_colour.to_s.start_with?('konsole_dodgerblue') and
         !use_this_colour.to_s.include?('colour_') )
        # ================================================================= #
        # In this case, rewrite the colour-code a little, such as from
        # 'konsole_dodgerblue' towards 'konsole_colour_blue'.
        # ================================================================= #
        unless use_this_colour.is_a? Symbol
          use_this_colour = 'konsole_colour_'+use_this_colour.to_s.split('_').last
        end
      end
      e ::Colours.send(use_this_colour, (use_this_token * 80))
    else
      if use_this_token.is_a? Symbol
        if ::Colours.respond_to?(use_this_token)
          use_this_token = ::Colours.send(use_this_token, '=')
        else
          use_this_token = ::Colours::HtmlColoursMethods.send(use_this_token, '=')
        end
      end
      e (use_this_token * 80)
    end 
  end
    
end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  require 'colours/html_colours'
  RBT.cliner
  e
  e 'And now with text:'
  e
  RBT.cliner { e 'Hi there' }
  e
  e 'And with colours:'
  e
  RBT.cliner('_', :steelblue)
  RBT.cliner('_', :steelblue) { e 'And hi again.' }
end # rbtcliner