#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/configure_appdir_prefix.rb'
# RBT.return_appdir_prefix('htop-2.3') # => "/home/Programs/Htop/2.3/"
# =========================================================================== #
module RBT

  begin
    require 'program_information'
  rescue LoadError; end

  require 'rbt/toplevel_methods/e_and_esystem.rb'
  require 'rbt/toplevel_methods/files_and_directories_related_methods.rb'

  # ========================================================================= #
  # === RBT.return_appdir_prefix
  #
  # This method assumes that the input includes at the least one
  # '-', so that ProgramInformation can work on this properly.
  #
  # Usage example:
  #
  #   RBT.return_appdir_prefix('htop-2.2.3')
  #
  # ========================================================================= #
  def self.return_appdir_prefix(
      of_this_program
    )
    if of_this_program.is_a? Array
      of_this_program = of_this_program.first
    end
    # ======================================================================= #
    # In the past, before September 2022, the method used .delete('-').
    # But this messed up ProgramInformation, so it was removed.
    # ======================================================================= #
    the_guessed_name = ProgramInformation.return_name(
                         of_this_program
                       ).capitalize
    return "#{programs_dir?}"\
           "#{the_guessed_name}/"\
           "#{ProgramInformation.return_version(of_this_program)}"\
           "/"
  end

  # ========================================================================= #
  # === RBT.configure_appdir_prefix
  #
  # This method will return the AppDir prefix, based on the given input,
  # which defaults to the current working directory.
  # ========================================================================= #
  def self.configure_appdir_prefix(
      i = return_pwd
    )
    i = i.first if i.is_a? Array
    if i.nil? or i.empty?
      i = return_pwd
    end
    target = File.basename(i)
    _ = './configure --prefix='+
        return_appdir_prefix(target)
    e _
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.configure_appdir_prefix(ARGV)
end # configureappdirprefix
    # configureappdirprefix htop-2.2.3