#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/determine_appdir_prefix_from_this_input.rb'
# RBT.determine_appdir_prefix_from_this_input
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/files_and_directories_related_methods.rb'
  require 'rbt/toplevel_methods/rds.rb'

  begin # Add the program_information gem - and a notification if it is not there.
    require 'program_information'
  rescue LoadError
    puts 'Please install the gem called `program_information`'
  end

  # ========================================================================= #
  # === RBT.determine_appdir_prefix_from_this_input
  #
  # This method will return a String such as "/Programs/Htop/1.1.1/"
  # or "/home/Programs/Htop/1.1.1/".
  #
  # Since as of August 2022, if the input includes a '/' then no
  # modification will be made. This is done primarily for
  # compatibility reasons.
  # ========================================================================= #
  def self.determine_appdir_prefix_from_this_input(
      name_of_the_program    = 'htop',
      version_of_the_program = nil
    )
    if name_of_the_program.is_a? Array
      name_of_the_program = name_of_the_program.first
    end
    name_of_the_program = 'htop' if name_of_the_program.nil?
    if name_of_the_program and name_of_the_program.include?('/')
      return name_of_the_program # Return it here in this case.
    end
    new_prefix = ''.dup
    # ======================================================================= #
    # Append the "/Programs/" string next, or whatever else the user
    # designated as his or her programs-directory:
    # ======================================================================= #
    new_prefix << programs_directory? 
    if name_of_the_program.nil?
      e 'Please supply a valid argument to this method.'
      e
      e 'A valid input-argument may be:'
      e
      e '  foobar-1.0'
      e
      raise 'A program-name (String) is required.' 
    end
    # ======================================================================= #
    # === Safeguard against improper versions
    # ======================================================================= #
    if version_of_the_program.nil?
      if name_of_the_program.include?('-')
        version_of_the_program = ProgramInformation.return_version(name_of_the_program)
        name_of_the_program = ProgramInformation.return_name(name_of_the_program)
      else
        version_of_the_program = 'Current/'
      end
    end
    new_prefix << name_of_the_program.capitalize
    new_prefix << '/'
    if version_of_the_program
      new_prefix << "#{version_of_the_program}/"
    end
    return rds(new_prefix)
  end; self.instance_eval { alias static_appdir_prefix_of? determine_appdir_prefix_from_this_input } # === RBT.static_appdir_prefix_of?

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  # e RBT.determine_appdir_prefix_from_this_input(ARGV)
  # e RBT.determine_appdir_prefix_from_this_input('htop')
  # e RBT.determine_appdir_prefix_from_this_input('htop', '2.3.4')
  e RBT.determine_appdir_prefix_from_this_input('htop-1.1.1')  # /home/Programs/Htop/1.1.1/
  e RBT.determine_appdir_prefix_from_this_input('cmake-3.5.2') # /home/Programs/Cmake/3.4.2/
end # rb determine_appdir_prefix*rb