#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/editor.rb'
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === @use_this_editor
  # ========================================================================= #
  @use_this_editor = nil

  # ========================================================================= #
  # === RBT::USE_THIS_EDITOR
  #
  # Here we can decide on which editor to use. This can be modified by
  # the user.
  # ========================================================================= #
  if ENV['EDITOR']
    @use_this_editor = ENV['EDITOR'].to_s.dup
  else # Else hardcode it. This is actually the default for my home system.
    @use_this_editor = 'bluefish'
  end

  # ========================================================================= #
  # === RBT.set_use_this_editor
  #
  # Setter-method for the default editor to be used.
  # ========================================================================= #
  def self.set_use_this_editor(i = :vim)
    @use_this_editor = i.to_s
  end

  # ========================================================================= #
  # === RBT.editor?
  # ========================================================================= #
  def self.editor?(
      i = @use_this_editor
    )
    return i
  end

end