#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/fast_return_file_size_of_this_program.rb'
# RBT.fast_return_file_size_of_this_program
# =========================================================================== #
module RBT

  require 'rbt/requires/require_yaml.rb'
  require 'rbt/toplevel_methods/e_and_esystem.rb'
  require 'rbt/configuration/configuration.rb'
  require 'rbt/constants/constants.rb'

  # ========================================================================= #
  # === RBT.fast_return_file_size_of_this_program
  #
  # This method will quickly return the file size of the given input
  # program. It relies on the expanded cookbook dataset for this.
  #
  # Invocation example from ruby:
  #
  #   RBT.fast_return_file_size_of_this_program(:htop) # => 213004
  #
  # ========================================================================= #
  def self.fast_return_file_size_of_this_program(
      i,
      use_this_as_database_backend = :use_the_expanded_cookbooks
    )
    use_this_as_the_key = :archive_size
    if i.is_a? Array
      i = i.join(' ').strip
    end
    i = i.to_s unless i.is_a? String
    case use_this_as_database_backend
    # ======================================================================= #
    # === :default
    #
    # This is the default entry point. We will use the expanded cookbooks.
    # ======================================================================= #
    when :default,
         :use_the_expanded_cookbooks
      base_dir_to_store_expanded_cookbooks = RBT.base_dir_to_store_expanded_cookbooks?
      if File.directory? base_dir_to_store_expanded_cookbooks
        # =================================================================== #
        # Obtain the path to the expanded-cookbook .yml file next.
        # =================================================================== #
        target_file = "#{base_dir_to_store_expanded_cookbooks}#{i}.yml"
        if File.exist? target_file
          dataset = YAML.load_file(target_file)
          archive_size = dataset[use_this_as_the_key]
          return archive_size
        else
          no_file_exists_at(target_file)
        end
      else
        e "#{rev}No directory exists at "\
          "#{base_dir_to_store_expanded_cookbooks}"
      end
    # ======================================================================= #
    # === :SQL
    #
    # This entry point was added in August 2022, to support sqlite databases.
    # ======================================================================= #
    when :SQL,
         :sql,
         :use_sqlite
      unless RBT.const_defined?(:CompileViaSQLDatabase)
        require 'rbt/requires/require_compile_via_sql_database.rb'
      end
      location_of_the_database = RBT::CompileViaSQLDatabase.location_of_the_database?
      result = `sqlite3 #{location_of_the_database} "select * from REGISTERED_PROGRAMS where PROGRAM_NAME = '#{i}';"`
      if result.include? '|'
        splitted = result.split('|')
        # The stored file size is at position 11.
        return splitted[11].to_i
      end
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  puts RBT.fast_return_file_size_of_this_program(ARGV)
end # fastreturnfilesizeofthisprogram htop