#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/install_libstdc_plus_plus.rb'
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/files_and_directories_related_methods.rb'

  # ========================================================================= #
  # === RBT.install_libstdc_plus_plus
  #
  # The following code was taken/adapted from:
  #
  #   http://www.linuxfromscratch.org/lfs/view/stable/chapter05/gcc-libstdc++.html
  #
  # ========================================================================= #
  def self.install_libstdc_plus_plus(
      shall_we_extract_the_tarball = true, # ← We could skip it, too.
      use_this_gcc_version         = :infer_automatically,
      to                           = RBT.log_dir?,
      use_this_as_target_prefix    = RBT.programs_directory?
    )
    require 'rbt/requires/require_extracter.rb'
    require 'rbt/toplevel_methods/e_and_esystem.rb'
    case use_this_gcc_version
    # ======================================================================= #
    # === :infer_automatically
    # ======================================================================= #
    when :infer_automatically, # or hardcode it: '10.1.0'
         :infer
      unless RBT.respond_to? :swift_return_version_of_this_program
        require 'rbt/requires/swift_version.rb'
      end
      use_this_gcc_version = RBT.swift_return_version_of_this_program(:gcc) 
    end
    what = "/home/x/src/gcc/gcc-#{use_this_gcc_version}.tar.xz"
    if shall_we_extract_the_tarball
      Extracter.what_to(what, to)
    end
    cd("#{to}#{use_this_gcc_version}/")
    use_this_as_host = 'x86_64-linux-gnu'
    target_dir = 'BUILD_FOR_LIBSTDC++'
    mkdir target_dir
    cd target_dir
    use_this_full_prefix_target = use_this_as_target_prefix+'Libstdc++/'+use_this_gcc_version
    _ = '../libstdc++-v3/configure           \
    --host='+use_this_as_host+'  \
    --prefix='+use_this_full_prefix_target+'                 \
    --disable-multilib              \
    --disable-nls                   \
    --disable-libstdcxx-threads     \
    --disable-libstdcxx-pch         \
    --with-gxx-include-dir='+use_this_as_target_prefix+'Gcc/Current/include/c++/'+use_this_gcc_version
    esystem _
    esystem 'make'
    esystem 'make install'
    action(:ToCurrent, use_this_full_prefix_target)
  end; self.instance_eval { alias install_libstdcpp install_libstdc_plus_plus } # === RBT.install_libstdcpp

end

if __FILE__ == $PROGRAM_NAME
  RBT.install_libstdc_plus_plus
end # installlibstdcplusplus