#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/install_which_binaries.rb'
# =========================================================================== #
module RBT

  require 'rbt/project/project.rb'
  require 'rbt/constants/constants.rb'
  require 'rbt/toplevel_methods/roebe.rb'
  require 'rbt/requires/require_raw_yaml.rb'

  # ========================================================================= #
  # === RBT.will_install_which_binaries?
  #
  # This method will simply return an Array of binaries that belong to
  # the rbt project. These all reside under the bin/ directory.
  # ========================================================================= #
  def self.will_install_which_binaries?
    results = []
    if is_on_roebe?
      results = Dir["#{RUBY_SRC_DIR_AT_HOME}rbt/bin/*"]
    else # Else we handle the case for regular users.
         # This depends on information stored in the file called
         # rbt_installs_these_executables.yml, which has to be
         # upgraded every now and then.
      results = YAML.load_file(
        "#{RBT.project_yaml_directory?}"\
        "rbt_installs_these_executables/rbt_installs_these_executables.yml"
      )
    end
    # ======================================================================= #
    # Use only the basename of any results:
    # ======================================================================= #
    results.map! {|entry|
      File.basename(entry)
    }
    results.sort # ← Keep the entries alphabetically sorted.
  end

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/autoinclude'
  _ = RBT.will_install_which_binaries?
  e
  e Colours.rev+'The following '+
    Colours.rev+
    Colours::HtmlColoursMethods.steelblue(_.size.to_s)+
    Colours.rev+' binaries will be installed by '+
    Colours.rev+
    Colours::HtmlColoursMethods.royalblue('RBT')+
    Colours.rev+':'
  e
  _.each {|entry|
    e "    #{Colours::HtmlColoursMethods.steelblue(entry)}"
  }
  e
end # installwhichbinaries