#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/md5sum.rb'
# =========================================================================== #
module RBT

  require 'rbt/requires/require_raw_yaml.rb'

  # ========================================================================= #
  # === @display_md5sum
  #
  # Need to set a default value for md5sum.
  # ========================================================================= #
  @display_md5sum = false

  # ========================================================================= #
  # === @display_md5sum
  #
  # The following instance variable will determine whether we will show
  # the md5sum or whether we will not.
  # ========================================================================= #
  if File.exist? FILE_USE_MD5SUM
    @display_md5sum = YAML.load_file(FILE_USE_MD5SUM)
  end

  # ========================================================================= #
  # === RBT.display_md5sum?
  #
  # Tell us whether we will display the md5sum values or whether we
  # will not.
  # ========================================================================= #
  def self.display_md5sum?
    @display_md5sum
  end

  # ========================================================================= #
  # === RBT.disable_md5sum_calculation
  #
  # Use this method to disable the md5sum calculation altogether.
  # ========================================================================= #
  def self.disable_md5sum_calculation
    @display_md5sum = false
  end; self.instance_eval { alias do_not_show_md5sum disable_md5sum_calculation } # === RBT.do_not_show_md5sum

  # ========================================================================= #
  # === RBT.enable_md5sum_calculation
  #
  # Use this method to disable the md5sum calculation altogether.
  # ========================================================================= #
  def self.enable_md5sum_calculation
    @display_md5sum = true
  end

end

if __FILE__ == $PROGRAM_NAME
  puts RBT.display_md5sum?
end # rb md5sum.rb