#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Code in this file is for toplevel-functionality related to the meson
# build system (and ninja).
#
# In particular it was necessary to quickly compile a certain program,
# such as atk, in an appdir-like approach, so the code elements here
# in this file were started in May 2019.
# =========================================================================== #
# require 'rbt/toplevel_methods/meson.rb'
# RBT.run_meson_configure_step
# RBT.meson_appdir_prefix
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/e_and_esystem.rb'
  require 'rbt/toplevel_methods/files_and_directories_related_methods.rb'

  # ========================================================================= #
  # === RBT.run_meson_configure_step
  #
  # This will simply run "meson".
  # ========================================================================= #
  def self.run_meson_configure_step(
      use_this_prefix          = '/usr/',
      use_this_build_directory = 'BUILD_DIRECTORY',
      &block
    )
    configure_options = ''.dup
    is_padded         = false
    use_colours       = false
    meson_command     = 'meson'.dup
    # ======================================================================= #
    # === Handle Hashes next
    # ======================================================================= #
    if use_this_prefix.is_a? Hash
      _ = use_this_prefix
      # ===================================================================== #
      # === :configure_options
      # ===================================================================== #
      if _.has_key?(:configure_options) and _[:configure_options] and
        !_[:configure_options].empty?
        configure_options << " #{_.delete(:configure_options)}"
      end
      # ===================================================================== #
      # === :use_this_as_build_directory
      # ===================================================================== #
      if _.has_key? :use_this_as_build_directory
        use_this_build_directory = _.delete(:use_this_as_build_directory)
      end
      # ===================================================================== #
      # The next entry point must come last.
      # ===================================================================== #
      if _.has_key? :prefix_to_use
        use_this_prefix = _.delete(:prefix_to_use).to_s
      end
    end
    # ======================================================================= #
    # Always use the absolute path here:
    # ======================================================================= #
    use_this_build_directory = File.absolute_path(use_this_build_directory)
    unless use_this_build_directory.end_with?('/')
      use_this_build_directory = use_this_build_directory.dup
      use_this_build_directory << '/'
    end
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :colourized_and_padded
      # ===================================================================== #
      when :colourized_and_padded
        is_padded = true
        use_colours = true
      end
    end
    meson_command.prepend('  ') if is_padded
    meson_command << " --prefix=#{use_this_prefix.to_s}"
    meson_command << " --libdir=#{use_this_prefix.to_s}"
    meson_command << configure_options
    meson_command << " #{use_this_build_directory}"
    if use_colours
      require 'rbt/colours/html_colours.rb'
      if meson_command and meson_command.include?('--prefix=') and use_colours
        meson_command = meson_command.dup
        # =================================================================== #
        # === Colourize the prefix, in cornflowerblue().
        # =================================================================== #
        colourized_command = meson_command.sub(/(--prefix=)(\/.+?)\s/, # See: https://rubular.com/r/Df8OzLqXGd
          olivedrab('\1')+
          cornflowerblue('\2 ')+
          ::Colours.remove_escape_sequences(teal(''))
        )
        colourized_command.sub!(/meson/, mediumaquamarine('meson'))
      end
    end
    e if is_padded
    if use_colours
      e colourized_command
      e if is_padded
      system meson_command
    else
      esystem meson_command
      e if is_padded
    end
  end

  # ========================================================================= #
  # === RBT.meson_appdir_prefix
  #
  # The argument to this method defaults to the symbol called
  # :use_cwd_as_input, which will be translated into the
  # current working directory.
  # ========================================================================= #
  def self.meson_appdir_prefix(
      i = :use_cwd_as_input
    )
    case i
    when :use_cwd_as_input
      i = return_pwd
    end
    i = File.basename(i)
    require 'program_information'
    this_program_name    = ProgramInformation.return_name(i).delete('-_')
    this_program_version = ProgramInformation.return_version(i)
    use_this_build_directory = 'BUILD_DIRECTORY_FOR_MESON'
    use_this_prefix = programs_dir?+
                      this_program_name.capitalize+
                      '/'+
                      this_program_version+
                      '/'
    esystem 'meson --prefix='+use_this_prefix+' '+use_this_build_directory
    cd use_this_build_directory
    esystem 'ninja'
    esystem 'ninja install'
    e "#{rev}If everything went fine then the program "\
      "#{sfancy(this_program_name)} #{rev}has been compiled"
    e "#{rev}successfully into the prefix `#{sfancy(use_this_prefix)}#{rev}`."
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.meson_appdir_prefix
end # mesonappdirprefix
