#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/module_programs.rb'
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/e_and_esystem.rb'

  # ========================================================================= #
  # === RBT.enable_module_programs
  #
  # This method will be invoked when we require the cookbooks project
  # in a special manner.
  #
  # We add the individual cookbooks onto the Cookbooks. namespace directly
  # there.
  #
  # This will then allow us to call methods such as:
  #
  #   RBT.htop
  #
  # Do note that we will also radically override already existing methods,
  # so this is functionality that should be used with care.
  #
  # Testing this method can happen either like this:
  #
  #   RBT.enable_module_programs; RBT.htop
  #   require 'rbt/module_programs'; htop = RBT.htop
  #   require 'rbt/toplevel_methods/module_programs.rb'; RBT.enable_module_programs; htop = RBT.htop.hash?
  #
  # ========================================================================= #
  def self.enable_module_programs(
      also_add_reader_methods = true
    )
    require 'rbt/requires/require_actions.rb'
    require 'rbt/requires/require_the_cookbook_class.rb'
    available_programs?.each {|this_program|
      self.class.class_eval {
        _ = this_program.to_sym
        if respond_to? _
          # This here happens when we enable the methods.
          # This is just here as a placeholder, if we ever want
          # to debug this.
        else
          # ::RBT.e 'Defining the method `'+_.to_s+'` next.' # <- This is more o a debug-statement.
          define_method(_) { # Simply delegate to class Cookbook then.
            action(:sanitize_cookbook, this_program, :fast)
          }
          # ================================================================= #
          # We will additionally add reader-methods, if the argument is
          # set to true. These reader methods will directly return the
          # dataset, as a Hash.
          #
          # This can be tested like so in IRB:
          #
          #   require 'rbt/module_programs'; x = RBT.htop.hash?
          #
          # ================================================================= #
          if also_add_reader_methods
            define_method("#{_}?") { # Simply delegate to class Cookbook then.
              action(:sanitize_cookbook, this_program, :fast).hash?
            }
          end
        end
      }
    }
  end

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/autoinclude'
  RBT.enable_module_programs
  e
  e sfancy('Now testing ')+
    Colours.slateblue('RBT.htop')+
    sfancy(':')
  e 
  pp RBT.htop
end # rbtmoduleprograms