#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/return_string_containing_all_program_versions.rb'
# RBT.return_string_containing_all_program_versions_the_last_update_and_the_remote_URL
# RBT.store_the_string_into_a_md_file
# =========================================================================== #
module RBT

  require 'rbt/configuration/initialize_configuration.rb'

  # ========================================================================= #
  # === RBT.return_string_containing_all_program_versions
  #
  # This method will return a string containing all programs versions
  # of the registered programs that are part of the RBT project. Do
  # note that the yaml file at hand has to exist, in order for the
  # code in this method to work properly.
  # ========================================================================= #
  def self.return_string_containing_all_program_versions
    file_location = RBT.file_programs_version
    if File.exist? file_location
      data = File.read(file_location)
      return data
    else
      no_file_exists_at(file_location)
      nil
    end
  end

  # ========================================================================= #
  # === RBT.return_string_containing_all_program_versions_the_last_update_and_the_remote_URL
  #
  # This method will return a much longer String, containing all program
  # versions of the registered programs, as well as when the last
  # update happened, and the remote URL.
  #
  # This depends on the expanded cookbooks dataset.
  #
  # The four entries should be:
  #
  #   (1) program name
  #   (2) program version
  #   (3) last update of this program (in regards to the cookbooks project)
  #   (4) remote URL
  #
  # ========================================================================= #
  def self.return_string_containing_all_program_versions_the_last_update_and_the_remote_URL
    require 'yaml'
    return_this_string = ''.dup
    string = return_string_containing_all_program_versions
    if string
      string.split(N).sort.each {|line|
        # =================================================================== #
        # The line will be like this:
        #
        #   zziplib: "0.13.66"
        #
        # =================================================================== #
        _ = line.split(':')
        program_name = _.first.delete('-')
        program_version = _.last.delete('"')
        this_file = "#{RBT.directory_expanded_cookbooks?}#{program_name}.yml"
        if File.exist? this_file
          dataset = YAML.load_file(this_file)
          last_update = dataset['last_update']
          remote_URL  = dataset['url1']
          return_this_string << 
            "#{program_name.ljust(28)} #{program_version.to_s.ljust(13)} #{last_update} #{remote_URL}\n"
        end
      }
      return return_this_string
    end
  end

  # ========================================================================= #
  # === RBT.store_the_string_into_a_md_file
  #
  # Note that this method will store into the log-directory.
  # ========================================================================= #
  def self.store_the_string_into_a_md_file
    RBT.initialize_configuration unless RBT.configuration_has_been_initialized_already?
    # e RBT.return_string_containing_all_program_versions
    string = RBT.return_string_containing_all_program_versions_the_last_update_and_the_remote_URL
    e string
    into = "#{RBT.log_dir?}programs_version_url.md"
    write_what_into(string, into)
    e "#{rev}Also stored into the file at `#{into}`."
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.store_the_string_into_a_md_file
end # returnstringcontainingallprogramversions