#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/try_to_find_shortcut_for_a_chain_compiled_program.rb'
# RBT.try_to_find_shortcut_for_a_chain_compiled_program
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === RBT.try_to_find_shortcut_for_a_chain_compiled_program
  #
  # Use this method if you wish to find a shortcut.
  # ========================================================================= #
  def self.try_to_find_shortcut_for_a_chain_compiled_program(
      i
    )
    if i.is_a? Array
      i = i.first # Only care about the first member of a given Array.
    end
    i = i.to_s.delete('_') # Since as of Aug 2011, we will get rid of all '_' characters.
    i = case i # case tag
    # ======================================================================= #
    # === kde
    # ======================================================================= #
    when 'kde',
         'kde3',
         'kde4',
         'kd',
         'default'
      'kde'
    # ======================================================================= #
    # === kde1
    #
    # This entry point is used as a shortcut for the programs listed in
    # KDE5 foundation.
    # ======================================================================= #
    when 'kde1',
         /kde5(_|-| )?foundation/
      'kde5_foundation'
    # ======================================================================= #
    # === xorg_libraries
    # ======================================================================= #
    when 'libs',
         'libraries',
         'xorg3',
         'xorglibraries',
         'x3',
         '3',
         /^-?-?compile(-|_)?xorg(-|_)?base$/i,
         /^-?-?xorg(-|_)?server(-|_)?libraries$/i
      'xorg_libraries'
    # ======================================================================= #
    # === games
    # ======================================================================= #
    when 'games',
         'gam'
      'games'
    # ======================================================================= #
    # === protos
    # ======================================================================= #
    when 'protos',
         'proto',
         'xorgprotos',
         'xorg1',
         'x1',
         '1'
      'xorg_protos'
    # ======================================================================= #
    # === utils
    # ======================================================================= #
    when 'utils',
         'xorgutils',
         'xorg2',
         'x2',
         '2'
      'xorg_utils'
    # ======================================================================= #
    # === data
    # ======================================================================= #
    when 'data',
         'xorg4',
         'xorgdata',
         'x4',
         '4'
      'xorg_data'
    # ======================================================================= #
    # === xorg_apps
    # ======================================================================= #
    when /xorg(_|-)?apps/,
         'apps',
         'xorg5',
         'x5',
         '5'
      'xorg_apps'
    # ======================================================================= #
    # === fonts
    # ======================================================================= #
    when 'fonts',
         'xorg6',
         'xorgfonts',
         'x6',
         '6'
      'xorg_fonts'
    # ======================================================================= #
    # === server
    # ======================================================================= #
    when 'server',
         'xorg7',
         'xorgserver',
         'x7',
         '7'
      'xorg_server'
    # ======================================================================= #
    # === addons
    # ======================================================================= #
    when 'addons',
         'ruby_addons',
         'rubyaddons',
         'raddons',
         'raddon',
         'rub',
         'r',
         'rand',
         'all',
         '8'
      'ruby_addons'
    # ======================================================================= #
    # === python
    # ======================================================================= #
    when 'python',
         'mypython',
         'pp',
         'pyt',
         'pyth',
         '9'
      'my_python'
    # ======================================================================= #
    # === gnome
    # ======================================================================= #
    when 'gnome',
         'xorg10',
         '10'
      'gnome5'
    # ======================================================================= #
    # === e
    # ======================================================================= #
    when 'e','11','xorg11','e17'
      'e17'
    # ======================================================================= #
    # === audio_suite
    # ======================================================================= #
    when 'audio_suite','audsuite'
      'aud_suite'
    when 'video_base','vidbase','vid_base',
         /video(_|-)?subsection/
      'vid_base'
    else
      i # Else return the input unaltered.
    end
    return i
  end

end

if __FILE__ == $PROGRAM_NAME
  puts RBT.try_to_find_shortcut_for_a_chain_compiled_program(ARGV)  
end # try_to_find_shortcut_for_a_chain_compiled_program kde1