#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/verbose_truth.rb'
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === RBT.verbose_truth
  #
  # This "converts" the given appropriate input to a "Yes." or "No."
  # String.
  #
  # If you need a boolean variant, you can use the method RBT.to_bool().
  #
  # By default a trailing '.' will be used by this method.
  # ========================================================================= #
  def self.verbose_truth(
      i                  = 'true',
      optional_arguments = nil
    )
    i = i.first if i.is_a? Array
    case i.to_s
    # ======================================================================= #
    # === Handle true cases
    # ======================================================================= #
    when 'true',
         't',
         'j',
         'ja',
         'yes',
         'y'
      i = 'Yes.'
    # ======================================================================= #
    # === Handle false cases
    # ======================================================================= #
    when 'false',
         'f',
         'no',
         'n'
      i = 'No.'
    end
    i = i.dup if i.frozen?
    if optional_arguments
      # ===================================================================== #
      # Handle cases such as this one here:
      #
      #   verbose_truth(is_on_windows?, :do_not_capitalize)
      #
      # ===================================================================== #
      if optional_arguments.is_a? Symbol
        case optional_arguments
        # =================================================================== #
        # === :do_not_capitalize
        # =================================================================== #
        when :do_not_capitalize
          i.downcase!
        end
      end
    end
    i
  end
  
end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e RBT.verbose_truth(ARGV)
end # ruby verbose_truth.rb y