#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::BinaryNameBelongsToWhichProgram
#
# This small class will report to which a particular program a specific
# binary belongs to. The name of the binary has to be used as main
# argument to this class.
#
# Usage example:
#
#   RBT::BinaryNameBelongsToWhichProgram.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/binary_name_belongs_to_which_program.rb'
# RBT::BinaryNameBelongsToWhichProgram.new
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class BinaryNameBelongsToWhichProgram < Base # === RBT::BinaryNameBelongsToWhichProgram

  require 'rbt/registered/registered_binaries.rb' # For RBT.hash_registered_binaries?.

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    first = first_argument?
    all_registered_binaries = RBT.hash_registered_binaries?
    if all_registered_binaries.has_key? first
      belongs_to = all_registered_binaries[first]
      opne "#{rev}The binary called `#{royalblue(first)}#{rev}` belongs to"
      opne "the program `#{orange(belongs_to)}#{rev}`."
    else
      opne "#{rev}The program called `#{royalblue(first)}#{rev}` does "\
           "not appear to be registered"
      opne 'with any particular program.'
    end
  end

  # ========================================================================= #
  # === RBT::BinaryNameBelongsToWhichProgram[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::BinaryNameBelongsToWhichProgram.new(ARGV)
end # binarynamebelongstowhichprogram 