#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Chainer
#
# This class may receive an argument, such as "ktpkdedmodule", and will
# then compile the whole compile-chain where this program is part of.
#
# For this to work, of course that particular program has to be
# registered; otherwise it will not work. In the event that the
# program is not registered, the user will be notified of this.
#
# The reason as to why this class was added is because I tend to not
# remember the name of the compile-chain, such as for kde-apps. So
# I can then simply do "chainer konsole", and all kde-apps will be
# compiled from source, including KDE Konsole.
#
# Simple usage example:
#
#   RBT::Chainer.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/chainer.rb'
# RBT::Chainer.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class Chainer < Base # === RBT::Chainer

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    run if run_already
  end
 
  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @dataset
    # ======================================================================= #
    @dataset = YAML.load_file(RBT.file_cookbook_chained_programs)
  end

  # ========================================================================= #
  # === is_this_program_included?
  # ========================================================================= #
  def is_this_program_included?(i = first_argument?)
    i = i.flatten if i.is_a? Array
    @dataset.values.flatten.include? i
  end

  # ========================================================================= #
  # === @dataset
  # ========================================================================= #
  def dataset?
    @dataset
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run 
    _ = first_argument?.to_s.downcase.delete('-')
    if is_this_program_included? _ 
      require 'rbt/requires/require_the_installer_class.rb'
      # ===================================================================== #
      # In this case we must select the proper key.
      # ===================================================================== #
      proper_hash = dataset?.select {|key, inner_array|
        key if inner_array.include? _
      }
      selected_programs = proper_hash.values.flatten
      selected_programs.each {|this_program|
        action(:Installer, this_program)
      }
    else
      notify_the_user_that_the_program_is_not_included(_)
    end
  end

  # ========================================================================= #
  # === notify_the_user_that_the_program_is_not_included
  # ========================================================================= #
  def notify_the_user_that_the_program_is_not_included(i)
    opne "The program #{steelblue(i)} is "\
         "#{crimson('NOT')} included."
  end

  # ========================================================================= #
  # === RBT::Chainer[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Chainer.new(ARGV)
end # chainer ktpkdedmodule
    # chainer libxmu
    # chainer kwaylandserver
    # chainer khotkeys