#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CompileIntoHomeDir
#
# This will compile the given input at hand into the home dir of
# the user.
#
# Usage example:
#
#   RBT::CompileIntoHomeDir.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/compile_into_home_dir.rb'
# RBT::CompileIntoHomeDir.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CompileIntoHomeDir < Base # === RBT::CompileIntoHomeDir

  require 'rbt/requires/require_the_installer_class.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_compile_these_programs(
      i
    )
    if block_given?
      yielded = yield
      set_commandline_arguments(yielded)
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    set_commandline_arguments
  end

  # ========================================================================= #
  # === set_compile_these_programs
  # ========================================================================= #
  def set_compile_these_programs(i)
    @compile_these_programs = [i].flatten.compact
    sanitize_compile_these_programs
  end

  # ========================================================================= #
  # === sanitize_compile_these_programs
  #
  # Currently this method will only replace numbers with the corresponding
  # entry from the file called "installation_procedure.yml" If that file
  # does not exist then this method will currently not do anything.
  # ========================================================================= #
  def sanitize_compile_these_programs
    _ = RBT.file_installation_procedere
    if File.exist? _
      dataset = YAML.load_file(_)['default']
      @compile_these_programs.map! {|entry|
        if entry and entry.match?(/^\d+$/)
          # ================================================================= #
          # Find the replacement next.
          # ================================================================= #
          entry = dataset[entry.to_i - 1]
        end
        entry
      }
    end
  end

  # ========================================================================= #
  # === compile_which_programs?
  # ========================================================================= #
  def compile_which_programs?
    @compile_these_programs
  end

  # ========================================================================= #
  # === without_symlinking?
  # ========================================================================= #
  def without_symlinking?
    commandline_arguments?.any? {|entry|
      entry == :without_symlinking
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    # ======================================================================= #
    # Instantiate our compile-object next, which will be adjusted properly.
    # ======================================================================= #
    _ = RBT::Action::Installer.new(:do_not_run_yet)
    compile_these_programs = compile_which_programs?
    compile_these_programs.each {|this_program|
      opne "Now attempting to compile the program `#{orange(this_program)}`."
      _.reset
      _.set_compile_this_program(this_program)
      _.prepare_dataset_for(this_program)
      _.set_user_prefix('$HOME') # Designate our prefix to HOME here.
      if without_symlinking? # Here we disallow symlinking.
        _.do_not_symlink { :be_verbose }
      end
      _.do_compile_this_program(this_program)
    }
  end

  # ========================================================================= #
  # === RBT::CompileIntoHomeDir[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::CompileIntoHomeDir.new(ARGV)
end # compileintohomedir sed