#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CompileVersionedProgram
#
# This class allows the user to invoke compilation based on simple input
# such as:
#
#   @1.74.0
#
# =========================================================================== #
# require 'rbt/utility_scripts/compile_versioned_program/compile_versioned_program.rb'
# RBT::CompileVersionedProgram
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CompileVersionedProgram < RBT::Base # === RBT::CompileVersionedProgram

  require 'rbt/requires/require_the_installer_class.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    # ======================================================================= #
    # === Handle blocks next:
    # ======================================================================= #
    if block_given?
      yielded = yield
      if yielded.is_a? Hash
        # =================================================================== #
        # === :use_this_directory
        # =================================================================== #
        if yielded.has_key? :use_this_directory
          set_use_this_directory(yielded.delete(:use_this_directory))
        end
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @target_directory
    # ======================================================================= #
    @target_directory = return_pwd
  end

  # ========================================================================= #
  # === set_target_directory
  # ========================================================================= #
  def set_target_directory(i)
    @target_directory = i
  end; alias set_use_this_directory set_target_directory # === set_use_this_directory

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    all_files = Dir[@target_directory+'**']
    work_on = first_argument?
    possible_matches = all_files.select {|entry|
      entry = File.basename(entry) # Get a shorter variant here.
      entry.include?(work_on)
    }
    if possible_matches.empty?
      opne "No match was found for the given input #{steelblue(work_on)}."
    else
      # ===================================================================== #
      # Next we have to find the proper match, from all
      # possible_matches determined above. Here we need
      # to be careful to find the correct entry.
      # ===================================================================== #
      _ = possible_matches.first
      assumed_name = File.basename(@target_directory).delete('-')
      assumed_version = ProgramInformation.return_version(_)
      _ = assumed_name+'@'+assumed_version
      set_first_commandline_argument(_)
      RBT::Action::Installer.new(commandline_arguments?)
    end
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::CompileVersionedProgram.new(ARGV)
end