#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::PurgeLibrariesOfThisProgram
#
# Use this class if you want to purge libraries in /usr/lib/ belonging
# to one particular program, such as "libx11".
#
# Be careful when using this - and be sure to really want to use
# it, before using it. Ideally also have backups available somewhere
# in case something is deleted that you did not want to delete.
#
# Usage example:
#
#   RBT::PurgeLibrariesOfThisProgram.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/purge_libraries_of_this_program.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class PurgeLibrariesOfThisProgram < Base # === RBT::PurgeLibrariesOfThisProgram

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    @cookbook = action(:SanitizeCookbook, input?) { :fast }
    all_libraries = @cookbook.libraries? # <- Obtain all registered binaries.
    if !all_libraries.empty?
      all_libraries.each {|file|
        target = "/usr/lib/#{File.basename(file)}"
        if File.exist? target
          opnn; remove_file(target, :be_verbose)
        end
      }
    end
  end

  # ========================================================================= #
  # === RBT::PurgeLibrariesOfThisProgram[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::PurgeLibrariesOfThisProgram.new(ARGV)
end # purgelibraries