#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Scons
#
# The task of class RBT:Scons is simple:
#
#   Contain all stuff that a "scons string" will need.
#
# This class is fairly incomplete, though, and may be
# subject to eventually removal (written this notice
# in August 2022).
# =========================================================================== #
# require 'rbt/utility_scripts/scons.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class Scons < Base # === RBT::Scons

  # ========================================================================= #
  # === USE_THIS_COMMAND
  # ========================================================================= #
  USE_THIS_COMMAND = 'scons'  # default command

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @scons_string
    # ======================================================================= #
    @scons_string   = "#{USE_THIS_COMMAND} ".dup
    # ======================================================================= #
    # === @default_prefix
    # ======================================================================= #
    @default_prefix = "#{programs_directory?}Libzub/0.2.3"
  end

  # ========================================================================= #
  # === default_prefix?
  #
  # The default prefix is stored here.
  # ========================================================================= #
  def default_prefix?
    @default_prefix
  end; alias default_prefix default_prefix? # === default_prefix

  # ========================================================================= #
  # === set_prefix
  #
  # set_prefix PROGRAMS_DIR+'/Mppenc/1.16/'
  # ========================================================================= #
  def set_prefix(
      to_this_prefix = @default_prefix
    )
    to_this_prefix = guess_prefix if to_this_prefix == :guess
    append 'PREFIX='+to_this_prefix
  end

  # ========================================================================= #
  # === append
  #
  # This method will simply append data onto the @scons_string. NO checking
  # involved, so call it only when you are sure that you want to add this.
  # ========================================================================= #
  def append(this_string)
    @scons_string << " #{this_string}"
  end

  # ========================================================================= #
  # === set_scons_lib_dir
  #
  # Not sure if this is still needed.
  # ========================================================================= #
  def set_scons_lib_dir
    _ = 'export SCONS_LIB_DIR=`pwd`/src/engine'
    opne _
    sys _
  end

  # ========================================================================= #
  # === pinstall
  #
  # Do some postinstall stuff here.
  # ========================================================================= #
  def pinstall
    system('python setup.py build')
    # sys 'python setup.py install'
    # sys 'python src/script/scons.py build/scons'
    # sys 'python setup.py install'
  end

  # ========================================================================= #
  # === guess_prefix
  #
  # of_this_dir should include a '-'
  # guess_prefix sets @default_prefix
  # ========================================================================= #
  def guess_prefix(
      of_this_dir = return_pwd
    )
    basename = File.basename(of_this_dir).split('-')
    @default_prefix = programs_directory?+
                      basename.first.to_s.capitalize+'/'+
                      basename[1]
    return @default_prefix
  end

  # ========================================================================= #
  # === rscons_string?
  #
  # Query method - the ivar @rscons_string contains the configure string
  # object.
  # ========================================================================= #
  def rscons_string?
    @rscons_string
  end; alias rscons_string rscons_string? # === rscons_string

  # ========================================================================= #
  # === run
  #
  # run it.
  # ========================================================================= #
  def run
    opne @scons_string
    system @scons_string
    ewarn 'Todo: do a scons install next?????????????? '\
          'DEBUG from '+sdir(return_pwd)
  end

end; end
                                                                                                          
if __FILE__ == $PROGRAM_NAME
  _ = RBT::Scons.new(ARGV, false)
  _.set_prefix :guess
  _.run
  _.pinstall
  RBT::AdvancedSymlink.new(_.default_prefix)
end # rscons