#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/utility_scripts/show_overview_over_available_utility_scripts.rb'
# =========================================================================== #
module RBT

  require 'rbt/constants/constants.rb'
  require 'rbt/toplevel_methods/cliner.rb'

  # ========================================================================= #
  # === RBT.show_overview_over_available_utility_scripts
  #
  # This method will show all available utility scripts and provide an
  # overview over them.
  #
  # To trigger this method, do:
  #
  #   cookbooks OVERVIEW
  #   cookbooks --show-available-utility-scripts
  #
  # ========================================================================= #
  def self.show_overview_over_available_utility_scripts
    begin
      require 'class_docu_shower'
    rescue LoadError; end
    target = "#{utility_scripts?}*.rb"
    available_files = Dir[target]
    if Object.const_defined? :ClassDocuShower
      available_files.each {|file|
        cliner
        ClassDocuShower[file] # Delegate towards ClassDocuShower here.
      }
    else
      opn(namespace: RBT::NAMESPACE[:namespace])
      e 'The class ClassDocuShower is not available.'
      e
      opn(namespace: RBT::NAMESPACE[:namespace])
      e 'It can be installed via:'
      e
      opn(namespace: RBT::NAMESPACE[:namespace])
      e '  gem install class_docu_shower'
      e
    end
  end; self.instance_eval { alias show_utility_scripts show_overview_over_available_utility_scripts } # === RBT.show_utility_scripts 

end

if __FILE__ == $PROGRAM_NAME
  RBT.show_overview_over_available_utility_scripts
end