#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SimplifiedLddOutput
#
# This class will create a large .yml file.
#
# Usage example:
#
#   RBT::SimplifiedLddOutput.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/simplified_ldd_output.rb'
# RBT.create_ldd_dataset
# RBT::SimplifiedLddOutput.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class SimplifiedLddOutput < Base # === RBT::SimplifiedLddOutput

  # ========================================================================= #
  # === initialize
  #
  # The input to this method should be a String such as
  # "/Programs/Python/3.6.2/".
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === :main_dataset
    # ======================================================================= #
    @internal_hash[:main_dataset] = {}
  end

  # ========================================================================= #
  # === main_dataset?
  # ========================================================================= #
  def main_dataset?
    @internal_hash[:main_dataset]
  end; alias hash? main_dataset? # === hash?

  # ========================================================================= #
  # === store_the_main_hash_into_a_yaml_file
  # ========================================================================= #
  def store_the_main_hash_into_a_yaml_file
    what = YAML.dump(hash?)
    into = rbt_log_dir+'sanitized_ldd_dataset.yml'
    opne "#{rev}Now storing into the file `#{sfile(into)}#{rev}`."
    write_what_into(what, into)
  end

  # ========================================================================= #
  # === parse_the_result_into_a_hash
  #
  # The String this method has to parse may look like this:
  #
  #   "/usr/lib/v4l2convert.so:\n" +
  #   "\tlinux-vdso.so.1 (0x00007ff525647000)\n" +
  #   "\tlibv4l2.so.0 => /usr/lib/libv4l2.so.0 (0x00007ff5255c0000)\n" +
  #
  # ========================================================================= #
  def parse_the_result_into_a_hash(result)
    _ = main_dataset? # Easer "pointer".
    @current_pointer = nil
    splitted = result.split(N)
    splitted.each {|entry|
      entry = entry.dup if entry.frozen?
      entry.rstrip! # Always rstrip it.
      if entry.include?(' (')
        entry = entry.split(' (').first.rstrip
      end
      if entry.start_with?("\t")
        modified_entry = entry.lstrip
        if modified_entry.include? '=>'
          modified_entry = modified_entry.split('=>').last.strip
        end
        if modified_entry.include?('not found') and is_on_roebe?
          e tomato('Warning: not found entry for ')+
            steelblue(
              entry.dup.delete(':')
            )+' ('+modified_entry+')'
          e 'This one will NOT be added.'
        else
          _[@current_pointer] << rds(modified_entry)
        end
      else
        @current_pointer = entry.dup.delete(':')
        _[@current_pointer] = []
      end
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    opne "#{rev}Checking all libraries at "+
         sdir('/usr/lib/')+
         "#{rev} next."
    result = `ldd /usr/lib/*.so* #{ERROR_LINE}`
    parse_the_result_into_a_hash(result)
    store_the_main_hash_into_a_yaml_file
  end

  # ========================================================================= #
  # === RBT::SimplifiedLddOutput[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end

# =========================================================================== #
# === RBT.simplified_ldd_output
# =========================================================================== #
def self.simplified_ldd_output(i = ARGV)
  RBT::SimplifiedLddOutput.new(i)
end; self.instance_eval { alias create_ldd_dataset simplified_ldd_output } # === RBT.create_ldd_dataset

# =========================================================================== #
# === RBT.file_ldd_dataset?
# =========================================================================== #
def self.file_ldd_dataset?
  "#{RBT.log_dir?}sanitized_ldd_dataset.yml"
end

# =========================================================================== #
# === RBT.ldd_dataset?
# =========================================================================== #
def self.ldd_dataset?
  YAML.load_file(RBT.file_ldd_dataset?)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.simplified_ldd_output(ARGV)
end # simplifiedlddoutput