#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SimplifyRootEntries
#
# This class can be used to replace all root:root entries in a
# given Makefile with 0:0 entries. The general idea here is to
# fix faulty Makefiles in an incomplete linux environment.
#
# Usage example:
#
#   RBT::SimplifyRootEntries.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/simplify_root_entries.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class SimplifyRootEntries < Base # === RBT::SimplifyRootEntries

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === work_on_input_file
  # ========================================================================= #
  def work_on_input_file
    _ = first_argument?
    if _ and File.file?(_)
      save_the_file = false
      dataset = File.readlines(_)
      index = 0
      dataset.map! {|line| index += 1
        if line.include? 'root:root'
          save_the_file = true
          line.gsub!(/root:root/, '0:0')
          opne "At line #{steelblue(index)} a root:root entry "\
               "has been replaced with 0:0."
        end
        line
      }
      if save_the_file
        into = _
        opne "Now saving the modified dataset into `#{sfile(into)}`."
        what = dataset.join
        write_what_into(what, into)
      end
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    work_on_input_file
  end

  # ========================================================================= #
  # === RBT::SimplifyRootEntries[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::SimplifyRootEntries.new(ARGV)
end # simplifyrootentries