#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/version/version.rb'
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === VERSION
  #
  # Which version does RBT have. Query via: RBT::VERSION. PROGRAM_VERSION is
  # just an alias to it - stick to using VERSION instead "officially"
  # whenever possible.
  # ========================================================================= #
  VERSION = '0.16.7'
    RBT_VERSION_NUMBER = VERSION # === RBT_VERSION_NUMBER
    PROGRAM_VERSION    = VERSION # === PROGRAM_VERSION

  # ========================================================================= #
  # === LAST_UPDATE
  #
  # This specifies when the RBT project was last updated. It should be
  # updated whenever the VERSION constant changes.
  # ========================================================================= #
  LAST_UPDATE = '28.03.2024' # ← Keep this constant in DD.MM.YYYY notation.

  # ========================================================================= #
  # === URL_TO_THE_DOCUMENTATION
  # ========================================================================= #
  URL_TO_THE_DOCUMENTATION =
    "https://www.rubydoc.info/gems/rbt/#{VERSION}"

  # ========================================================================= #
  # === RBT.version?
  # ========================================================================= #
  def self.version?
    RBT::VERSION
  end; self.instance_eval { alias rbt_version  version? } # === RBT.rbt_version()
       self.instance_eval { alias rbt_version? version? } # === RBT.rbt_version?()

  # ========================================================================= #
  # === rbt_version?
  # ========================================================================= #
  def rbt_version?
    RBT.version?
  end

  # ========================================================================= #
  # === RBT.last_update?
  # ========================================================================= #
  def self.last_update?
    LAST_UPDATE
  end

  # ========================================================================= #
  # === RBT.report_rbt_version
  #
  # This method will simply report the current RBT-version in use.
  # ========================================================================= #
  def self.report_rbt_version(
      optional_additional_activities = nil
    )
    require 'rbt/toplevel_methods/e_and_esystem.rb'
    require 'rbt/colours/rev_simp_sdir_sfancy_sfile_and_swarn.rb'
    string = rbt_version?
    if optional_additional_activities
      case optional_additional_activities
      when :also_show_when_the_project_was_last_updated
        string = string.dup
        string << " (Last update: #{LAST_UPDATE})"
      end
    end
    e "#{sfancy(string)}#{rev}"
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.report_rbt_version  
end # rbt_report_rbt_version